clear all

disp('DC-motor initialization...')

%constants

global L J R_nom K_T_nom K_E_nom

% Data for DC motor (Maxon RE 36.118800)
L = 0.487e-3; %mH -> H
R = 2.74; %Omega
K_T = 56.6e-3; %mNm/A -> Nm/A
K_E = K_T; %V/(rad/s)
J = 6.78e-6; %g cm^2 -> kg m^2

R_nom = R;
K_T_nom = K_T;
K_E_nom = K_E;

%J = J*1.10; % Assume a somewhat higher inertia (connected gears etc.)

% Sampling frequency
global Ts
Ts = 1e-3;
fs = 1/Ts; % 1 kHz

% Encoder resolution
counts_per_revolution = 2000;
quantizer_interval = 2*pi/counts_per_revolution;

% Butterworth filter
f_c = 100; % Hz
omega_c = 2*pi*f_c; % rad/s
nyquist = fs/2;
[B,A] = butter(2,omega_c,'s');
[Ab,Bb,Cb,Db] = butter(2,omega_c,'s');
sys = ss(Ab,Bb,Cb,Db);
sysd = c2d(sys,Ts,'zoh');

f_cc = 25;
omega_cc = 2*pi*f_cc;
[Bc,Ac] = butter(5,omega_cc,'s'); % Smoothing for the resistance att. in the discrete model

% PD-gains
omega_n = 27*2*pi;
psi = 0.7;
K_p = (J*omega_n^2)/K_T;
K_d = (2*psi*omega_n*J)/K_T;

K_i = 1;

% Resistance attenuation parameters
as = 12.5/R; % Start at 15 Ohm (R*(as + cs))
bs = -5;
cs = 2.5/R; % End at 2.5 Ohm (R*(cs))

% Noise
noise_var = 0.005^2; % Noise variance
%noise_var = 0;

% Estimator paramters
h = Ts;
my = 2*0.01;
gamma = -1/h*my;
G_0 = 1/(R*(as+cs));

% Resistance gain scheduler parameters
a = 12.6314;
b = -4.9843;
c = 2.4553;
d = 0.3105;

% Spring and damper constants
i_max = 0.7;
K_spring = (i_max*K_T)/400; % Reach i_max*K_T = 0.04 Nm after 400 radians
K_damper = K_spring*1e-1; % Viscous friction

% Trajectory
alpha_max = 400;
omega_max = 40;
T = [0 0.1 10.1 10.2 15 15.1 25.1 25.2 30 30.1 50.1 50.2 55]'; %Time (s)
V = [0 40  40   0    0  40   40   0    0  -40  -40  -0.0 0]'; %Velocity (rad/s)
