#include <math.h>
#include <string.h>
#include "kalman_R_K_ib.h"
#include "kalman_R_K_ib_prm.h"

void MdlEnable(void)
{
  
  rtDWork.Sine_Wave_a_IWORK.SystemEnable = (int_T) TRUE;
  
  rtDWork.Sine_Wave_b_IWORK.SystemEnable = (int_T) TRUE;
}

void MdlInitialize(void)
{
  
  rtDWork.Difference_DSTATE = 0.0;

  {
    SimStruct *rts = ssGetSFunction(rtS, 0);
    sfcnInitializeConditions(rts);
  }
  
  rtDWork.Thermal_model_Kalman_DSTATE[0] = rtP.Thermal_model_Kalman_X0[0];
  rtDWork.Thermal_model_Kalman_DSTATE[1] = rtP.Thermal_model_Kalman_X0[1];

  rtDWork.Discrete_Butterworth_DSTATE[0] = rtP.Discrete_Butterworth_X0;
  rtDWork.Discrete_Butterworth_DSTATE[1] = rtP.Discrete_Butterworth_X0;

  rtDWork.iMeas_k_1_DSTATE = rtP.iMeas_k_1_X0;

  rtDWork.G_k_1_DSTATE = rtP.G_k_1_X0;

  rtDWork.vel_k_1_DSTATE = rtP.vel_k_1_X0;

  rtDWork.pwm_k_1_DSTATE = rtP.pwm_k_1_X0;
}

void MdlStart(void)
{
  
  rtDWork.From_Workspace_IWORK.PrevIndex = 0;

  rtB.Constant_a = rtP.Constant_a_Value;

  rtB.Constant1_a = rtP.Constant1_a_Value;

  rtB.Constant_b = rtP.Constant_b_Value;

  rtB.Constant1_b = rtP.Constant1_b_Value;

  {
    MQ3_encResetAll(0x320);
    MQ3_encSetCount(0x320, (uint_T)rtP.Encoder_Input_P1,
     (int_T)rtP.Encoder_Input_P2);
  }

  rtB.Bias_a = rtP.Bias_a_Value;

  {
    int16_T value;

    value = MQ_dacVoltageToOutput(rtP.Analog_Output_P2);
    MQ_dacOutput(0x320, (uint_T)rtP.Analog_Output_P1, value);
  }

  {
    MQ_adcInitialize(0x320);
  }

  rtB.A_D_offset_Volt = rtP.A_D_offset_Volt_Value;

  rtB.Closed_loop_gain = rtP.Closed_loop_gain_Value;

  rtB.Input_offset_Volt = rtP.Input_offset_Volt_Value;

  rtB.Bias_b = rtP.Bias_b_Value;

  rtB.Kelvin_a = rtP.Kelvin_a_Value;

  rtB.T_offset_K = rtP.T_offset_K_Value;

  rtB.Kelvin_b = rtP.Kelvin_b_Value;

  rtB.Ambient_temperature = rtP.Ambient_temperature_Value;

  rtB.iMeas_k_1 = rtP.iMeas_k_1_X0;

  rtB.G_k_1 = rtP.G_k_1_X0;

  rtB.vel_k_1 = rtP.vel_k_1_X0;

  rtB.pwm_k_1 = rtP.pwm_k_1_X0;

  rtB.Constant_c = rtP.Constant_c_Value;

  rtB.Constant1_c = rtP.Constant1_c_Value;

  rtB.Constant_d = rtP.Constant_d_Value;

  rtB.Constant1_d = rtP.Constant1_d_Value;

  MdlInitialize();
  MdlEnable();
}

void MdlOutputs(int_T tid)
{
  
  (void)tid;

  {
    real_T t = ssGetT(rtS);
    real_T *pTimeValues = (real_T *) rtDWork.From_Workspace_PWORK.TimePtr;
    real_T *pDataValues = (real_T *) rtDWork.From_Workspace_PWORK.DataPtr;
    if (t < pTimeValues[0]) {
      rtB.From_Workspace = 0.0;
    } else if (t >= pTimeValues[10199]) {
      rtB.From_Workspace = pDataValues[10199];
    } else {
      int_T currTimeIndex = rtDWork.From_Workspace_IWORK.PrevIndex;
      if (t < pTimeValues[currTimeIndex]) {
        while (t < pTimeValues[currTimeIndex]) {
          currTimeIndex--;
        }
      } else {
        while (t >= pTimeValues[currTimeIndex + 1]) {
          currTimeIndex++;
        }
      }
      rtB.From_Workspace = pDataValues[currTimeIndex];
      rtDWork.From_Workspace_IWORK.PrevIndex = currTimeIndex;
    }
  }
  if (ssGetT(rtS) < rtP.Step_a_Time) {
    rtB.Step_a = rtP.Step_a_Y0;
  } else {
    rtB.Step_a = rtP.Step_a_YFinal;
  }

  rtB.Clock_a = ssGetT(rtS);

  rtB.Constant_a = rtP.Constant_a_Value;

  rtB.Sum_a = rtB.Clock_a - rtB.Constant_a;

  rtB.Product_a = rtB.Step_a * rtB.Sum_a;

  rtB.Constant1_a = rtP.Constant1_a_Value;

  rtB.Sum1_a = rtB.Product_a + rtB.Constant1_a;

  if (ssGetT(rtS) < rtP.Step_b_Time) {
    rtB.Step_b = rtP.Step_b_Y0;
  } else {
    rtB.Step_b = rtP.Step_b_YFinal;
  }

  rtB.Clock_b = ssGetT(rtS);

  rtB.Constant_b = rtP.Constant_b_Value;

  rtB.Sum_b = rtB.Clock_b - rtB.Constant_b;

  rtB.Product_b = rtB.Step_b * rtB.Sum_b;

  rtB.Constant1_b = rtP.Constant1_b_Value;

  rtB.Sum1_b = rtB.Product_b + rtB.Constant1_b;

  rtB.Sum2_a = rtB.Sum1_a - rtB.Sum1_b;

  if ( (int_T)rtDWork.Sine_Wave_a_IWORK.SystemEnable ) {
    rtDWork.Sine_Wave_a_RWORK.LastSin = sin(rtP.Sine_Wave_a_Freq * ssGetT(rtS));
    rtDWork.Sine_Wave_a_RWORK.LastCos = cos(rtP.Sine_Wave_a_Freq * ssGetT(rtS));
    rtDWork.Sine_Wave_a_IWORK.SystemEnable = (int_T)FALSE;
  }

  rtB.Sine_Wave_a =
    rtP.Sine_Wave_a_Amp *
    (
    (rtDWork.Sine_Wave_a_RWORK.LastSin * rtP.Sine_Wave_a_PCos
    + rtDWork.Sine_Wave_a_RWORK.LastCos * rtP.Sine_Wave_a_PSin) *
    rtP.Sine_Wave_a_HCos
    + (rtDWork.Sine_Wave_a_RWORK.LastCos * rtP.Sine_Wave_a_PCos
    - rtDWork.Sine_Wave_a_RWORK.LastSin * rtP.Sine_Wave_a_PSin) *
    rtP.Sine_Wave_a_Hsin
    ) + rtP.Sine_Wave_a_Bias;

  rtB.Pos_a = rtB.Sum2_a * rtB.Sine_Wave_a;

  rtB.Sum1_c = rtB.From_Workspace + rtB.Pos_a;

  {

    rtB.Encoder_Input = MQ3_encInput(0x320, (uint_T)rtP.Encoder_Input_P1);
  }

  rtB.rad_count = rtB.Encoder_Input * rtP.rad_count_Gain;

  rtB.ZOH_3 = rtB.rad_count;

  rtB.Sum4_a = rtB.Sum1_c - rtB.ZOH_3;

  rtB.K_p = rtB.Sum4_a * rtP.K_p_Gain;

  rtB.Bias_a = rtP.Bias_a_Value;

  rtB.Sum_c = rtB.K_p - rtB.Bias_a;

  rtB.Amplification = rtB.Sum_c * rtP.Amplification_Gain;

  if (rtB.Amplification >= rtP.Sat_a_UpperSat) {
    rtB.Sat_a = rtP.Sat_a_UpperSat;
  } else if (rtB.Amplification <= rtP.Sat_a_LowerSat) {
    rtB.Sat_a = rtP.Sat_a_LowerSat;
  } else {
    rtB.Sat_a = rtB.Amplification;
  }

  {
    int16_T value;

    value = MQ_dacVoltageToOutput(rtB.Sat_a);
    MQ_dacOutput(0x320, (uint_T)rtP.Analog_Output_P1, value);
  }

  {
    int_T value;

    value = MQ_adcInput(0x320, (uint_T)rtP.Current_P1);
    rtB.Current = MQ_adcInputToVoltage(value);
  }

  {
    int_T value;

    value = MQ_adcInput(0x320, (uint_T)rtP.Current_filtered_P1);
    rtB.Current_filtered = MQ_adcInputToVoltage(value);
  }

  rtB.K_curr = rtB.Current * rtP.K_curr_Gain;

  rtB.ZOH_1 = rtB.K_curr;

  rtB.K_curr_filt = rtB.Current_filtered * rtP.K_curr_filt_Gain;

  rtB.ZOH_2 = rtB.K_curr_filt;

  rtB.Difference = rtP.Difference_D*rtB.ZOH_3;
  rtB.Difference += rtP.Difference_C*rtDWork.Difference_DSTATE;
  rtB.Ts = rtB.Difference * rtP.Ts_Gain;

  {
    int_T value;

    value = MQ_adcInput(0x320, (uint_T)rtP.AD595_output_P1);
    rtB.AD595_output = MQ_adcInputToVoltage(value);
  }

  rtB.A_D_offset_Volt = rtP.A_D_offset_Volt_Value;

  rtB.Closed_loop_gain = rtP.Closed_loop_gain_Value;

  rtB.Input_offset_Volt = rtP.Input_offset_Volt_Value;

  rtB.Bias_b = rtP.Bias_b_Value;

  if (rtB.K_p >= rtP.Sat_b_UpperSat) {
    rtB.Sat_b = rtP.Sat_b_UpperSat;
  } else if (rtB.K_p <= rtP.Sat_b_LowerSat) {
    rtB.Sat_b = rtP.Sat_b_LowerSat;
  } else {
    rtB.Sat_b = rtB.K_p;
  }

  rtB.Sum1_d = - rtB.Bias_b + rtB.Sat_b;

  rtB.TmpHiddenBuf1_Driving_Kalman_R[0] = rtB.ZOH_2;
  rtB.TmpHiddenBuf1_Driving_Kalman_R[1] = rtB.ZOH_3;

  {
    SimStruct *rts = ssGetSFunction(rtS, 0);
    sfcnOutputs(rts, tid);
  }

  rtB.V_A = rtB.Kalman_R[0] * rtP.V_A_Gain;

  rtB.Sum4_b = rtB.AD595_output - rtB.V_A;

  rtB.Sum1_e = - rtB.A_D_offset_Volt + rtB.Sum4_b;

  rtB.Resiprocal_a = 1.0/(rtB.Closed_loop_gain);

  rtB.Product_c = rtB.Sum1_e * rtB.Resiprocal_a;

  rtB.Sum_d = rtB.Product_c - rtB.Input_offset_Volt;

  rtB.V_to_mV = rtB.Sum_d * rtP.V_to_mV_Gain;

  rtB.Kelvin_a = rtP.Kelvin_a_Value;

  rtB.S_Function = rtP.S_Function_P1;

  rtB.Type_K_mV_to_Celsius = rt_Lookup(rtP.Type_K_mV_to_Celsius_XData, 207,
    rtB.V_to_mV, rtP.Type_K_mV_to_Celsius_YData);

  rtB.Type_T_mV_to_Celsius = rt_Lookup(rtP.Type_T_mV_to_Celsius_XData, 209,
    rtB.V_to_mV, rtP.Type_T_mV_to_Celsius_YData);

  if (rtB.S_Function) {
    rtB.Switch = rtB.Type_K_mV_to_Celsius;
  } else {
    rtB.Switch = rtB.Type_T_mV_to_Celsius;
  }

  rtB.T_linear_error_comp_1 = rtB.Switch * rtP.T_linear_error_comp_1_Gain;

  rtB.T_offset_K = rtP.T_offset_K_Value;

  rtB.Sum1_f = rtB.T_linear_error_comp_1 - rtB.T_offset_K;

  rtB.Sum2_b = rtB.Sum1_f + rtB.Kelvin_a;

  rtB.Square = rtB.Kalman_R[0] * rtB.Kalman_R[0];

  rtB.Qjoule = rtB.Kalman_R[6] * rtB.Square;

  rtB.Kelvin_b = rtP.Kelvin_b_Value;

  rtB.Ambient_temperature = rtP.Ambient_temperature_Value;

  {
    rtB.Thermal_model_Kalman[0] = rtP.Thermal_model_Kalman_D[0]*rtB.Sum2_b;
    rtB.Thermal_model_Kalman[0] +=
      rtP.Thermal_model_Kalman_C[0]*rtDWork.Thermal_model_Kalman_DSTATE[1];

    rtB.Thermal_model_Kalman[1] = rtP.Thermal_model_Kalman_D[1]*rtB.Sum2_b;
    rtB.Thermal_model_Kalman[1] +=
      rtP.Thermal_model_Kalman_C[1]*rtDWork.Thermal_model_Kalman_DSTATE[0]
      +rtP.Thermal_model_Kalman_C[2]*rtDWork.Thermal_model_Kalman_DSTATE[1];

    rtB.Thermal_model_Kalman[2] = rtP.Thermal_model_Kalman_D[2]*rtB.Sum2_b;
    rtB.Thermal_model_Kalman[2] +=
      rtP.Thermal_model_Kalman_C[3]*rtDWork.Thermal_model_Kalman_DSTATE[1];
  }

  rtB.Sum2_c[0] = - rtB.Kelvin_b + rtB.Thermal_model_Kalman[0];
  rtB.Sum2_c[1] = - rtB.Kelvin_b + rtB.Thermal_model_Kalman[1];
  rtB.Sum2_c[2] = - rtB.Kelvin_b + rtB.Thermal_model_Kalman[2];

  rtB.Sum7 = - rtB.Kelvin_b + rtB.Sum2_b;

  {
    rtB.Discrete_Butterworth =
      rtP.Discrete_Butterworth_C*rtDWork.Discrete_Butterworth_DSTATE[1];
  }

  rtB.iMeas_k_1 = rtDWork.iMeas_k_1_DSTATE;

  rtB.G_k_1 = rtDWork.G_k_1_DSTATE;

  rtB.i_hat = rtB.Discrete_Butterworth * rtB.G_k_1;

  rtB.Sum1_g = rtB.iMeas_k_1 - rtB.i_hat;

  rtB.error_u = rtB.Discrete_Butterworth * rtB.Sum1_g;

  rtB.mu = rtB.error_u * rtP.mu_Gain;

  rtB.G_k = rtB.mu + rtB.G_k_1;

  rtB.Resiprocal_b = 1.0/(rtB.G_k);

  rtB.vel_k_1 = rtDWork.vel_k_1_DSTATE;

  rtB.K_E = rtB.vel_k_1 * rtP.K_E_Gain;

  rtB.pwm_k_1 = rtDWork.pwm_k_1_DSTATE;

  rtB.Sum_e = - rtB.K_E + rtB.pwm_k_1;

  rtB.Sum3 = rtB.ZOH_2 - rtB.Kalman_R[8];

  if (ssGetT(rtS) < rtP.Step_c_Time) {
    rtB.Step_c = rtP.Step_c_Y0;
  } else {
    rtB.Step_c = rtP.Step_c_YFinal;
  }

  rtB.Clock_c = ssGetT(rtS);

  rtB.Constant_c = rtP.Constant_c_Value;

  rtB.Sum_f = rtB.Clock_c - rtB.Constant_c;

  rtB.Product_d = rtB.Step_c * rtB.Sum_f;

  rtB.Constant1_c = rtP.Constant1_c_Value;

  rtB.Sum1_h = rtB.Product_d + rtB.Constant1_c;

  if (ssGetT(rtS) < rtP.Step_d_Time) {
    rtB.Step_d = rtP.Step_d_Y0;
  } else {
    rtB.Step_d = rtP.Step_d_YFinal;
  }

  rtB.Clock_d = ssGetT(rtS);

  rtB.Constant_d = rtP.Constant_d_Value;

  rtB.Sum_g = rtB.Clock_d - rtB.Constant_d;

  rtB.Product_e = rtB.Step_d * rtB.Sum_g;

  rtB.Constant1_d = rtP.Constant1_d_Value;

  rtB.Sum1_i = rtB.Product_e + rtB.Constant1_d;

  rtB.Sum2_e = rtB.Sum1_h - rtB.Sum1_i;

  if ( (int_T)rtDWork.Sine_Wave_b_IWORK.SystemEnable ) {
    rtDWork.Sine_Wave_b_RWORK.LastSin = sin(rtP.Sine_Wave_b_Freq * ssGetT(rtS));
    rtDWork.Sine_Wave_b_RWORK.LastCos = cos(rtP.Sine_Wave_b_Freq * ssGetT(rtS));
    rtDWork.Sine_Wave_b_IWORK.SystemEnable = (int_T)FALSE;
  }

  rtB.Sine_Wave_b =
    rtP.Sine_Wave_b_Amp *
    (
    (rtDWork.Sine_Wave_b_RWORK.LastSin * rtP.Sine_Wave_b_PCos
    + rtDWork.Sine_Wave_b_RWORK.LastCos * rtP.Sine_Wave_b_PSin) *
    rtP.Sine_Wave_b_HCos
    + (rtDWork.Sine_Wave_b_RWORK.LastCos * rtP.Sine_Wave_b_PCos
    - rtDWork.Sine_Wave_b_RWORK.LastSin * rtP.Sine_Wave_b_PSin) *
    rtP.Sine_Wave_b_Hsin
    ) + rtP.Sine_Wave_b_Bias;

  rtB.Pos_b = rtB.Sum2_e * rtB.Sine_Wave_b;
}

void MdlUpdate(int_T tid)
{
  
  (void)tid;

  {
    real_T hold_sin;
    real_T hold_cos;

    hold_sin = rtDWork.Sine_Wave_a_RWORK.LastSin;
    hold_cos = rtDWork.Sine_Wave_a_RWORK.LastCos;

    rtDWork.Sine_Wave_a_RWORK.LastSin = hold_sin * rtP.Sine_Wave_a_HCos +
      hold_cos * rtP.Sine_Wave_a_Hsin;
    rtDWork.Sine_Wave_a_RWORK.LastCos = hold_cos * rtP.Sine_Wave_a_HCos -
      hold_sin * rtP.Sine_Wave_a_Hsin;
  }

  {
    rtDWork.Difference_DSTATE = rtB.ZOH_3
      +rtP.Difference_A*rtDWork.Difference_DSTATE;
  }

  {
    static real_T xnew[2];
    xnew[0] =
      rtP.Thermal_model_Kalman_B[0]*rtB.Qjoule+rtP.Thermal_model_Kalman_B[1]*rtB.Ambient_temperature
      +rtP.Thermal_model_Kalman_B[2]*rtB.Sum2_b;
    xnew[0] +=
      rtP.Thermal_model_Kalman_A[0]*rtDWork.Thermal_model_Kalman_DSTATE[0]
      +rtP.Thermal_model_Kalman_A[1]*rtDWork.Thermal_model_Kalman_DSTATE[1];

    xnew[1] =
      rtP.Thermal_model_Kalman_B[3]*rtB.Qjoule+rtP.Thermal_model_Kalman_B[4]*rtB.Ambient_temperature
      +rtP.Thermal_model_Kalman_B[5]*rtB.Sum2_b;
    xnew[1] +=
      rtP.Thermal_model_Kalman_A[2]*rtDWork.Thermal_model_Kalman_DSTATE[0]
      +rtP.Thermal_model_Kalman_A[3]*rtDWork.Thermal_model_Kalman_DSTATE[1];
    (void)memcpy(&rtDWork.Thermal_model_Kalman_DSTATE[0], xnew,
     sizeof(real_T)*2);
  }

  {
    static real_T xnew[2];
    xnew[0] = rtP.Discrete_Butterworth_B[0]*rtB.Sum_e;
    xnew[0] +=
      rtP.Discrete_Butterworth_A[0]*rtDWork.Discrete_Butterworth_DSTATE[0]
      +rtP.Discrete_Butterworth_A[1]*rtDWork.Discrete_Butterworth_DSTATE[1];

    xnew[1] = rtP.Discrete_Butterworth_B[1]*rtB.Sum_e;
    xnew[1] +=
      rtP.Discrete_Butterworth_A[2]*rtDWork.Discrete_Butterworth_DSTATE[0]
      +rtP.Discrete_Butterworth_A[3]*rtDWork.Discrete_Butterworth_DSTATE[1];
    (void)memcpy(&rtDWork.Discrete_Butterworth_DSTATE[0], xnew,
     sizeof(real_T)*2);
  }

  rtDWork.iMeas_k_1_DSTATE = rtB.Sum3;

  rtDWork.G_k_1_DSTATE = rtB.G_k;

  rtDWork.vel_k_1_DSTATE = rtB.Ts;

  rtDWork.pwm_k_1_DSTATE = rtB.Sum1_d;

  {
    real_T hold_sin;
    real_T hold_cos;

    hold_sin = rtDWork.Sine_Wave_b_RWORK.LastSin;
    hold_cos = rtDWork.Sine_Wave_b_RWORK.LastCos;

    rtDWork.Sine_Wave_b_RWORK.LastSin = hold_sin * rtP.Sine_Wave_b_HCos +
      hold_cos * rtP.Sine_Wave_b_Hsin;
    rtDWork.Sine_Wave_b_RWORK.LastCos = hold_cos * rtP.Sine_Wave_b_HCos -
      hold_sin * rtP.Sine_Wave_b_Hsin;
  }
}

void MdlTerminate(void)
{

  {
    int16_T value;

    value = MQ_dacVoltageToOutput(rtP.Analog_Output_P3);
    MQ_dacOutput(0x320, (uint_T)rtP.Analog_Output_P1, value);
  }

  {
    SimStruct *rts = ssGetSFunction(rtS, 0);
    sfcnTerminate(rts);
  }
}

#include "kalman_R_K_ib_reg.h"

#include "WinConLibrary.h"

void MdlRegister( void )
{
  
  {
    WCRegisterVariable("kalman_R_K_ib\vAD595", &rtB.V_to_mV);
    WCRegisterScope("kalman_R_K_ib\vAD595", 1
     , &rtB.V_to_mV
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vK", &rtB.Kalman_R[7]);
    WCRegisterScope("kalman_R_K_ib\vK", 1
     , &rtB.Kalman_R[7]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vQ", &rtB.Qjoule);
    WCRegisterScope("kalman_R_K_ib\vQ", 1
     , &rtB.Qjoule
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vR", &rtB.Kalman_R[6]);
    WCRegisterScope("kalman_R_K_ib\vR", 1
     , &rtB.Kalman_R[6]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vRef", &rtB.Sum1_c);
    WCRegisterScope("kalman_R_K_ib\vRef", 1
     , &rtB.Sum1_c
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vT_L", &rtB.Kalman_R[5]);
    WCRegisterScope("kalman_R_K_ib\vT_L", 1
     , &rtB.Kalman_R[5]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vTh", &rtB.Sum2_c[2]);
    WCRegisterScope("kalman_R_K_ib\vTh", 1
     , &rtB.Sum2_c[2]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vTh_meas", &rtB.Sum7);
    WCRegisterScope("kalman_R_K_ib\vTh_meas", 1
     , &rtB.Sum7
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vTw", &rtB.Sum2_c[1]);
    WCRegisterScope("kalman_R_K_ib\vTw", 1
     , &rtB.Sum2_c[1]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vi", &rtB.Kalman_R[0]);
    WCRegisterScope("kalman_R_K_ib\vi", 1
     , &rtB.Kalman_R[0]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vi_meas", &rtB.ZOH_1);
    WCRegisterScope("kalman_R_K_ib\vi_meas", 1
     , &rtB.ZOH_1
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vib", &rtB.Kalman_R[8]);
    WCRegisterScope("kalman_R_K_ib\vib", 1
     , &rtB.Kalman_R[8]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vif", &rtB.Kalman_R[2]);
    WCRegisterScope("kalman_R_K_ib\vif", 1
     , &rtB.Kalman_R[2]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vif_meas", &rtB.ZOH_2);
    WCRegisterScope("kalman_R_K_ib\vif_meas", 1
     , &rtB.ZOH_2
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vmWinv", &rtB.Resiprocal_b);
    WCRegisterScope("kalman_R_K_ib\vmWinv", 1
     , &rtB.Resiprocal_b
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vomega", &rtB.Kalman_R[3]);
    WCRegisterScope("kalman_R_K_ib\vomega", 1
     , &rtB.Kalman_R[3]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vtheta", &rtB.Kalman_R[4]);
    WCRegisterScope("kalman_R_K_ib\vtheta", 1
     , &rtB.Kalman_R[4]
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vtheta_meas", &rtB.ZOH_3);
    WCRegisterScope("kalman_R_K_ib\vtheta_meas", 1
     , &rtB.ZOH_3
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vu", &rtB.Sum1_d);
    WCRegisterScope("kalman_R_K_ib\vu", 1
     , &rtB.Sum1_d
     );
  }

  {
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vFrom Workspace",
     &rtB.From_Workspace);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp\vStep", &rtB.Step_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp\vClock", &rtB.Clock_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp\vConstant",
     &rtB.Constant_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp\vSum", &rtB.Sum_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp\vProduct",
     &rtB.Product_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp\vConstant1",
     &rtB.Constant1_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp\vSum1", &rtB.Sum1_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp1\vStep", &rtB.Step_b);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp1\vClock", &rtB.Clock_b);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp1\vConstant",
     &rtB.Constant_b);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp1\vSum", &rtB.Sum_b);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp1\vProduct",
     &rtB.Product_b);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp1\vConstant1",
     &rtB.Constant1_b);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vRamp1\vSum1", &rtB.Sum1_b);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vSum2", &rtB.Sum2_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vSine Wave", &rtB.Sine_Wave_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vPos", &rtB.Pos_a);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref.\vSum1", &rtB.Sum1_c);
    WCRegisterVariable("kalman_R_K_ib\vInput\vEncoder Input",
     &rtB.Encoder_Input);
    WCRegisterVariable("kalman_R_K_ib\vInput\vrad//count", &rtB.rad_count);
    WCRegisterVariable("kalman_R_K_ib\vInput\vZOH 3", &rtB.ZOH_3);
    WCRegisterVariable("kalman_R_K_ib\vController\vSum4", &rtB.Sum4_a);
    WCRegisterVariable("kalman_R_K_ib\vController\vK_p", &rtB.K_p);
    WCRegisterVariable("kalman_R_K_ib\vOutput\vBias", &rtB.Bias_a);
    WCRegisterVariable("kalman_R_K_ib\vOutput\vSum", &rtB.Sum_c);
    WCRegisterVariable("kalman_R_K_ib\vOutput\vAmplification",
     &rtB.Amplification);
    WCRegisterVariable("kalman_R_K_ib\vOutput\vSat", &rtB.Sat_a);
    WCRegisterVariable("kalman_R_K_ib\vInput\vCurrent", &rtB.Current);
    WCRegisterVariable("kalman_R_K_ib\vInput\vCurrent filtered",
     &rtB.Current_filtered);
    WCRegisterVariable("kalman_R_K_ib\vInput\vK_curr", &rtB.K_curr);
    WCRegisterVariable("kalman_R_K_ib\vInput\vZOH 1", &rtB.ZOH_1);
    WCRegisterVariable("kalman_R_K_ib\vInput\vK_curr_filt", &rtB.K_curr_filt);
    WCRegisterVariable("kalman_R_K_ib\vInput\vZOH 2", &rtB.ZOH_2);
    WCRegisterVariable("kalman_R_K_ib\vInput\vDifference", &rtB.Difference);
    WCRegisterVariable("kalman_R_K_ib\vInput\vTs", &rtB.Ts);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 output",
     &rtB.AD595_output);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 transfer function\vA//D offset [Volt]",
     &rtB.A_D_offset_Volt);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 transfer function\vClosed loop gain",
     &rtB.Closed_loop_gain);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 transfer function\vInput offset [Volt]",
     &rtB.Input_offset_Volt);
    WCRegisterVariable("kalman_R_K_ib\vBias", &rtB.Bias_b);
    WCRegisterVariable("kalman_R_K_ib\vSat", &rtB.Sat_b);
    WCRegisterVariable("kalman_R_K_ib\vSum1", &rtB.Sum1_d);
    WCRegisterVariable("kalman_R_K_ib\vTmpHiddenBuf1_Driving_Kalman, R_AtInput2[0]",
     &rtB.TmpHiddenBuf1_Driving_Kalman_R[0] + 0);
    WCRegisterVariable("kalman_R_K_ib\vTmpHiddenBuf1_Driving_Kalman, R_AtInput2[1]",
     &rtB.TmpHiddenBuf1_Driving_Kalman_R[0] + 1);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[0]", &rtB.Kalman_R[0] + 0);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[1]", &rtB.Kalman_R[0] + 1);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[2]", &rtB.Kalman_R[0] + 2);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[3]", &rtB.Kalman_R[0] + 3);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[4]", &rtB.Kalman_R[0] + 4);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[5]", &rtB.Kalman_R[0] + 5);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[6]", &rtB.Kalman_R[0] + 6);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[7]", &rtB.Kalman_R[0] + 7);
    WCRegisterVariable("kalman_R_K_ib\vKalman, R[8]", &rtB.Kalman_R[0] + 8);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\v[V//A]", &rtB.V_A);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vSum4", &rtB.Sum4_b);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 transfer function\vSum1",
     &rtB.Sum1_e);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 transfer function\vResiprocal",
     &rtB.Resiprocal_a);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 transfer function\vProduct",
     &rtB.Product_c);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 transfer function\vSum",
     &rtB.Sum_d);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vAD595 transfer function\v[V] to [mV]",
     &rtB.V_to_mV);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vKelvin", &rtB.Kelvin_a);

    WCRegisterVariable("kalman_R_K_ib\vTemperature\vType K mV to Celsius",
     &rtB.Type_K_mV_to_Celsius);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vType T mV to Celsius",
     &rtB.Type_T_mV_to_Celsius);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vManual Switch\vSwitch",
     &rtB.Switch);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vT linear error comp. [1]",
     &rtB.T_linear_error_comp_1);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vT offset [K]",
     &rtB.T_offset_K);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vSum1", &rtB.Sum1_f);
    WCRegisterVariable("kalman_R_K_ib\vTemperature\vSum2", &rtB.Sum2_b);
    WCRegisterVariable("kalman_R_K_ib\vSquare", &rtB.Square);
    WCRegisterVariable("kalman_R_K_ib\vQjoule", &rtB.Qjoule);
    WCRegisterVariable("kalman_R_K_ib\vKelvin", &rtB.Kelvin_b);
    WCRegisterVariable("kalman_R_K_ib\vAmbient temperature",
     &rtB.Ambient_temperature);
    WCRegisterVariable("kalman_R_K_ib\vThermal model, Kalman[0]",
     &rtB.Thermal_model_Kalman[0] + 0);
    WCRegisterVariable("kalman_R_K_ib\vThermal model, Kalman[1]",
     &rtB.Thermal_model_Kalman[0] + 1);
    WCRegisterVariable("kalman_R_K_ib\vThermal model, Kalman[2]",
     &rtB.Thermal_model_Kalman[0] + 2);
    WCRegisterVariable("kalman_R_K_ib\vSum2[0]", &rtB.Sum2_c[0] + 0);
    WCRegisterVariable("kalman_R_K_ib\vSum2[1]", &rtB.Sum2_c[0] + 1);
    WCRegisterVariable("kalman_R_K_ib\vSum2[2]", &rtB.Sum2_c[0] + 2);
    WCRegisterVariable("kalman_R_K_ib\vSum7", &rtB.Sum7);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vDiscrete Butterworth",
     &rtB.Discrete_Butterworth);
    WCRegisterVariable("kalman_R_K_ib\vLMS\viMeas(k-1)", &rtB.iMeas_k_1);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vG(k-1)", &rtB.G_k_1);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vi_hat", &rtB.i_hat);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vSum1", &rtB.Sum1_g);
    WCRegisterVariable("kalman_R_K_ib\vLMS\verror*u", &rtB.error_u);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vmu", &rtB.mu);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vG(k)", &rtB.G_k);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vResiprocal", &rtB.Resiprocal_b);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vvel(k-1)", &rtB.vel_k_1);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vK_E", &rtB.K_E);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vpwm(k-1)", &rtB.pwm_k_1);
    WCRegisterVariable("kalman_R_K_ib\vLMS\vSum", &rtB.Sum_e);
    WCRegisterVariable("kalman_R_K_ib\vSum3", &rtB.Sum3);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp\vStep", &rtB.Step_c);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp\vClock", &rtB.Clock_c);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp\vConstant",
     &rtB.Constant_c);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp\vSum", &rtB.Sum_f);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp\vProduct",
     &rtB.Product_d);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp\vConstant1",
     &rtB.Constant1_c);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp\vSum1", &rtB.Sum1_h);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp1\vStep", &rtB.Step_d);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp1\vClock", &rtB.Clock_d);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp1\vConstant",
     &rtB.Constant_d);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp1\vSum", &rtB.Sum_g);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp1\vProduct",
     &rtB.Product_e);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp1\vConstant1",
     &rtB.Constant1_d);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vRamp1\vSum1", &rtB.Sum1_i);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vSum2", &rtB.Sum2_e);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vSine Wave",
     &rtB.Sine_Wave_b);
    WCRegisterVariable("kalman_R_K_ib\vPos. ref. 2\vPos", &rtB.Pos_b);
  }

  {
    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp\vStep\vTime",
     &rtP.Step_a_Time);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp\vStep\vBefore",
     &rtP.Step_a_Y0);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp\vStep\vAfter",
     &rtP.Step_a_YFinal);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp\vConstant\vValue",
     &rtP.Constant_a_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp\vConstant1\vValue",
     &rtP.Constant1_a_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp1\vStep\vTime",
     &rtP.Step_b_Time);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp1\vStep\vBefore",
     &rtP.Step_b_Y0);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp1\vStep\vAfter",
     &rtP.Step_b_YFinal);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp1\vConstant\vValue",
     &rtP.Constant_b_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vRamp1\vConstant1\vValue",
     &rtP.Constant1_b_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vSine Wave\vAmplitude",
     &rtP.Sine_Wave_a_Amp);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vSine Wave\vBias",
     &rtP.Sine_Wave_a_Bias);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref.\vSine Wave\vFrequency",
     &rtP.Sine_Wave_a_Freq);

    WCRegisterScalarParameter("kalman_R_K_ib\vInput\vEncoder Input\vChannels",
     &rtP.Encoder_Input_P1);

    WCRegisterScalarParameter("kalman_R_K_ib\vInput\vEncoder Input\vInitialValues",
     &rtP.Encoder_Input_P2);

    WCRegisterScalarParameter("kalman_R_K_ib\vInput\vrad//count\vGain",
     &rtP.rad_count_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vController\vK_p\vGain",
     &rtP.K_p_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vOutput\vBias\vValue",
     &rtP.Bias_a_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vOutput\vAmplification\vGain",
     &rtP.Amplification_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vOutput\vSat\vUpperLimit",
     &rtP.Sat_a_UpperSat);

    WCRegisterScalarParameter("kalman_R_K_ib\vOutput\vSat\vLowerLimit",
     &rtP.Sat_a_LowerSat);

    WCRegisterScalarParameter("kalman_R_K_ib\vOutput\vAnalog Output\vChannels",
     &rtP.Analog_Output_P1);

    WCRegisterScalarParameter("kalman_R_K_ib\vOutput\vAnalog Output\vInitialOutputs",
     &rtP.Analog_Output_P2);

    WCRegisterScalarParameter("kalman_R_K_ib\vOutput\vAnalog Output\vFinalOutputs",
     &rtP.Analog_Output_P3);

    WCRegisterScalarParameter("kalman_R_K_ib\vInput\vCurrent\vChannels",
     &rtP.Current_P1);

    WCRegisterScalarParameter("kalman_R_K_ib\vInput\vCurrent filtered\vChannels",
     &rtP.Current_filtered_P1);

    WCRegisterScalarParameter("kalman_R_K_ib\vInput\vK_curr\vGain",
     &rtP.K_curr_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vInput\vK_curr_filt\vGain",
     &rtP.K_curr_filt_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vInput\vTs\vGain", &rtP.Ts_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\vAD595 output\vChannels",
     &rtP.AD595_output_P1);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\vAD595 transfer function\vA//D offset [Volt]\vValue",
     &rtP.A_D_offset_Volt_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\vAD595 transfer function\vClosed loop gain\vValue",
     &rtP.Closed_loop_gain_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\vAD595 transfer function\vInput offset [Volt]\vValue",
     &rtP.Input_offset_Volt_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vBias\vValue", &rtP.Bias_b_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vSat\vUpperLimit",
     &rtP.Sat_b_UpperSat);

    WCRegisterScalarParameter("kalman_R_K_ib\vSat\vLowerLimit",
     &rtP.Sat_b_LowerSat);

    WCRegisterVectorParameter("kalman_R_K_ib\vKalman, R\vP1", 9,
     &rtP.Kalman_R_P1[0]);

    WCRegisterVectorParameter("kalman_R_K_ib\vKalman, R\vP2", 9,
     &rtP.Kalman_R_P2[0]);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\v[V//A]\vGain",
     &rtP.V_A_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\vAD595 transfer function\v[V] to [mV]\vGain",
     &rtP.V_to_mV_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\vKelvin\vValue",
     &rtP.Kelvin_a_Value);

    WCRegisterVectorParameter("kalman_R_K_ib\vTemperature\vType K mV to Celsius\vInputValues",
     207, &rtP.Type_K_mV_to_Celsius_XData[0]);

    WCRegisterVectorParameter("kalman_R_K_ib\vTemperature\vType K mV to Celsius\vOutputValues",
     207, &rtP.Type_K_mV_to_Celsius_YData[0]);

    WCRegisterVectorParameter("kalman_R_K_ib\vTemperature\vType T mV to Celsius\vInputValues",
     209, &rtP.Type_T_mV_to_Celsius_XData[0]);

    WCRegisterVectorParameter("kalman_R_K_ib\vTemperature\vType T mV to Celsius\vOutputValues",
     209, &rtP.Type_T_mV_to_Celsius_YData[0]);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\vT linear error comp. [1]\vGain",
     &rtP.T_linear_error_comp_1_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vTemperature\vT offset [K]\vValue",
     &rtP.T_offset_K_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vKelvin\vValue",
     &rtP.Kelvin_b_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vAmbient temperature\vValue",
     &rtP.Ambient_temperature_Value);

    WCRegisterVectorParameter("kalman_R_K_ib\vThermal model, Kalman\vX0", 2,
     &rtP.Thermal_model_Kalman_X0[0]);

    WCRegisterScalarParameter("kalman_R_K_ib\vLMS\vDiscrete Butterworth\vX0",
     &rtP.Discrete_Butterworth_X0);

    WCRegisterScalarParameter("kalman_R_K_ib\vLMS\viMeas(k-1)\vX0",
     &rtP.iMeas_k_1_X0);

    WCRegisterScalarParameter("kalman_R_K_ib\vLMS\vG(k-1)\vX0", &rtP.G_k_1_X0);

    WCRegisterScalarParameter("kalman_R_K_ib\vLMS\vmu\vGain", &rtP.mu_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vLMS\vvel(k-1)\vX0",
     &rtP.vel_k_1_X0);

    WCRegisterScalarParameter("kalman_R_K_ib\vLMS\vK_E\vGain", &rtP.K_E_Gain);

    WCRegisterScalarParameter("kalman_R_K_ib\vLMS\vpwm(k-1)\vX0",
     &rtP.pwm_k_1_X0);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp\vStep\vTime",
     &rtP.Step_c_Time);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp\vStep\vBefore",
     &rtP.Step_c_Y0);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp\vStep\vAfter",
     &rtP.Step_c_YFinal);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp\vConstant\vValue",
     &rtP.Constant_c_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp\vConstant1\vValue",
     &rtP.Constant1_c_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp1\vStep\vTime",
     &rtP.Step_d_Time);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp1\vStep\vBefore",
     &rtP.Step_d_Y0);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp1\vStep\vAfter",
     &rtP.Step_d_YFinal);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp1\vConstant\vValue",
     &rtP.Constant_d_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vRamp1\vConstant1\vValue",
     &rtP.Constant1_d_Value);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vSine Wave\vAmplitude",
     &rtP.Sine_Wave_b_Amp);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vSine Wave\vBias",
     &rtP.Sine_Wave_b_Bias);

    WCRegisterScalarParameter("kalman_R_K_ib\vPos. ref. 2\vSine Wave\vFrequency",
     &rtP.Sine_Wave_b_Freq);
  }
}

void MdlChecksum( uint_T iValues[4] )
{

  iValues[0] = 687560014U;
  iValues[1] = 3365923026U;
  iValues[2] = 2580296478U;
  iValues[3] = 1172401556U;
}

uint_T MdlGetNumDataTypes()
{
  return 13;
}

boolean_T MdlGetDataTypeSizes(uint_T *puSizes, uint_T cDataTypes)
{
  
  
  
  

  if (cDataTypes == 13)
  {
    memcpy(puSizes, rtDataTypeSizes, cDataTypes * sizeof(uint_T));
    return 1; 
  }

  return 0; 
}

real_T MdlSamplingPeriod( void )
{
  return(0.0005);
}

