#include "matrix_operations.h"

// Format and print a matrix to stdio.
static void print_matrix(float **A, int N, int M)
{
    int i, j;
    for (i = 1; i <= N; i++) {
        for (j = 1; j <= M; j++) {
            printf("%e\t", A[i][j]);
        }
        printf("\n");
    }
	return;
}

// Copy (sub)matrix, for convenience -> check indexing: A (n x m) -> B (n x m), rows n to N and columns m to M
static void copy_matrix(float **A, float **B, int n, int N, int m, int M)
{
    int i, j;
    for (i = n; i <= N; i++) {
        for (j = m; j <= M; j++) {
            B[i][j] = A[i][j];
        }
    }
}

// Add two matrices with the same dimensions: A (n x m), B (n x m) -> C (n x m)
static void matrix_addition(float **A, float **B, float **C, int N, int M)
{
    int i, j;
    for (i = 1; i <= N; i++) {
        for (j = 1; j <= M; j++) {
            C[i][j] = A[i][j] + B[i][j];
        }
    }
}

// Subtract two matrices with the same dimensions: A (n x m), B (n x m) -> C (n x m)
static void matrix_subtraction(float **A, float **B, float **C, int N, int M)
{
    int i, j;
    for (i = 1; i <= N; i++) {
        for (j = 1; j <= M; j++) {
            C[i][j] = A[i][j] - B[i][j];
        }
    }
}

// Multiply two matrices: A (n x p), B (p x m) -> C (n x m)
static void matrix_multiply(float **A, float **B, float **C, int N, int P, int M)
{
    int i, j, k;
    for (i = 1; i <= N; i++) { // for all columns of A
        for (j = 1; j <= M; j++) { // for all rows of B
            C[i][j] = 0.0;
            for (k = 1; k <= P; k++) {
                C[i][j] += A[i][k]*B[k][j];
            }
        }
    }
}

// Transpose a matrix: A (n x m) -> B (m x n)
static void matrix_transpose(float **A, float **B, int N, int M)
{
    int i, j;
    for (i = 1; i <= N; i++) {
        for (j = 1; j <= M; j++) {
            B[j][i] = A[i][j];
        }
    }
}

// Initialize a matrix to all zeros
static void init_zeros(float **A, int N, int M)
{
    int i, j;
    for (i = 1; i <= N; i++) {
        for (j = 1; j <= M; j++) {
            A[i][j] = 0.0F;
        }
    }
}

// Initialize a square matrix to the indetity matrix
static void init_ident(float **A, int N)
{
    int i, j;
    for (i = 1; i <= N; i++) {
        for (j = 1; j <= N; j++) {
            if (i == j) {
                A[i][j] = 1.0F;
            } else {
                A[i][j] = 0.0F;
            }
        }
    }
}
