% Runs a given Simulink model with WinCon.

% Clear all variables
if 1
    clear all
end

% Set/reset some configuration parameters
if 1
    set_params
end

% Init, set up needed variables, LTI models etc.
init

if 0
    model = 'kalman_R_ib';
    project_name = 'kalman_R_ib'; % 60 sec. buffer
    %project_name = 'kalman_R_ib_long_buffer'; % 300 sec. buffer
    %project_name = 'kalman_R_ib_short_buffer'; % 15 sec. buffer
else
    model = 'comparison';
    project_name = 'comparison';
end

path = wc_path(model);

if wc_open(project_name)
    error(['Could not find WinCon project: ' model]);
end

% Make a clean build, usually not necessary
if 0
    wc_clean(model) 
end

% Build model before running, to be safe -> do after variables have changed
if 1
    wc_setoptions
    wc_build(model, path)
end

%plot_names = {'u', 'i', 'if', 'omega', 'theta', 'T_L', 'R', 'ib', 'i_meas', 'if_meas', 'theta_meas', 'mWinv', 'Th_meas', 'Th', 'Tw'};
%plot_names{16} = 'AD595';
plot_names = {'u', 'i', 'if', 'omega', 'theta', 'T_L', 'R', 'ib', 'i_meas', 'if_meas', 'theta_meas', 'mWinv', 'GS', 'ir'};


% Make a directory to save the files in
dname = datestr(now, 30);
mkdir(['data\' dname]);

% Save experiment parameters
save(['data\' dname '\parameters'], 'dir', 'SinusFrq', 'SinusAmp', 'Wref', 'Weight', 'T_L_approx', 'Tambient', 'Thousing', 'N_runs');

% Update initial values for thermal model
DT = toc;
Tinit(1) = (Tinit(1) - Tinit(2))*exp(-DT/tau1) + Tinit(2);
disp(Tinit - 273.15)

for n = 1:N_runs
    fprintf('Run # %d:\n', n);
    
    wc_start(model, path);
    fprintf('\tRunning...\n');
    pause( wait_time ); % Wait a while...
    wc_stop( model, path );
    tic % Time operations between stop and start
    wc_stop( model, path );
    
    % Save data to a file with a timestamp in the filname 
    fname_root = 'data';
    
    % Save a file for each timeseries
    for i = 1:length(plot_names)
        
        fname = ['data\' dname '\' fname_root '-' plot_names{i} '-' int2str(n) '.mat'];
        wc_saveplot(['Scope - ' model '\' plot_names{i}], fname);
        
        % Wait for the data to be saved...
        fprintf('\tSaving data to ''%s'': ', fname);
        done = 0;
        while (done == 0)
            pause(0.5);
            if (exist(fname, 'file') == 2)
                done = 1;
            end
        end
        fprintf('\nData saved.\n')
    end
    
    % Check saved files to see if data was saved, WinCon is weird sometimes...
    for i = 1:length(plot_names)
        fname = ['data\' dname '\' fname_root '-' plot_names{i} '-' int2str(n) '.mat'];
        S = load(fname);
        if ~eval(['length(S.' model '_' plot_names{i} ')'])
            error('Something wrong with the saved data file...')
        end
    end
    
    % Update initial values for thermal model
    if 0
        fname = ['data\' dname '\' fname_root '-' plot_names{14} '-' int2str(n) '.mat'];
        S = load(fname);
        Tinit(2) = (mean(S.kalman_R_ib_Th(end-500:end)) + 273.15); % Housing
        
        fname = ['data\' dname '\' fname_root '-' plot_names{15} '-' int2str(n) '.mat'];
        S = load(fname);
        Tinit(1) = (mean(S.kalman_R_ib_Tw(end-500:end)) + 273.15); % Winding
        DT = toc;
        Tinit(1) = (Tinit(1) - Tinit(2))*exp(-DT/tau1) + Tinit(2); % Account for temp. decay while saving files etc.
        disp(Tinit - 273.15)
    end
end

% Disconnect from the real-time code
wc_disconnect( model, path );


