% Thermal model
% Celsius to Kelvin: K = C + 273.15 
% The states are Tw and Th - winding and housing temperatures.
% The inputs are Q and Tinf - Joule losses (electrical power loss) and
% ambient temperature.

tau1 =   44.2; % s
tau2 = 1120.0; % s

Rth1 = 3.4; % K/W
Rth2 = 6.4; % K/W

Cth1 = tau1/Rth1;
Cth2 = tau2/Rth2;

Etherm = diag([Cth1 Cth2]);

Ftherm = [-1/Rth1, 1/Rth1; 1/Rth1, -(1/Rth1 + 1/Rth2)]; 
Gtherm = [1, 0; 0, 1/Rth2];

Atherm = inv(Etherm)*Ftherm;
Btherm = inv(Etherm)*Gtherm;

Ctherm = [0, 1]; % Th (housing temperature) is measured
Dtherm = zeros(1,2); % No feedthrough

% Kalman filter, state observer
Gw = [1 0; 0 1];
Hw = [0 0];

Qn = [0.10 0; 0 0.25]; % Process noise
Rn = [1.00]; % Measurement noise

sys_therm = ss(Atherm, [Btherm Gw], Ctherm, [Dtherm Hw]);
% ZOH discretization
sys_therm_d = c2d(sys_therm, Ts);

KEST = kalman(sys_therm_d, Qn, Rn);

load('type_k_coefficients'); % Type K thermocouple e -> T coefficients
load('type_t_coefficients'); % Type T thermocouple e -> T coefficients
