% Processes data from WinCon.
function [vltg,crnt,vlct,rstnc,trq,tmprtr] = process_data_v2(S,n)

%  1 - 'u'
%  2 - 'i'
%  3 - 'if'
%  4 - 'omega'
%  5 - 'theta'
%  6 - 'T_L'
%  7 - 'R'
%  8 - 'ib'
%  9 - 'i_meas'
% 10 - 'if_meas'
% 11 - 'theta_meas'
% 12 - 'mWinv'
% 13 - 'Th_meas'
% 14 - 'Th'
% 15 - 'Tw'

clear Q R Vrms Irms vol cur vel res trq tmp

TRUNCATE = 1;
if TRUNCATE
    Ts = 5e-4; % sec
    DT = 15; % sec
    L = DT/Ts;
end
    
P = 1; % Voltage
for k = 1:n
    if TRUNCATE
        u.ts{k} = S{P,k}.kalman_R_ib_u(end-L:end); % Time series
    else
        u.ts{k} = S{P,k}.kalman_R_ib_u; % Time series
    end
    u.rms(k) = sqrt(mean(u.ts{k}.^2)); % Root mean square
end

P = 2; % Current (armature)
for k = 1:n
    if TRUNCATE
        i.ts{k} = S{P,k}.kalman_R_ib_i(end-L:end);
    else
        i.ts{k} = S{P,k}.kalman_R_ib_i;
    end
    i.rms(k) = sqrt(mean(i.ts{k}.^2));
end

P = 4; % Angular velocity
for k = 1:n
    if TRUNCATE
        omega.ts{k} = S{P,k}.kalman_R_ib_omega(end-L:end);
    else
        omega.ts{k} = S{P,k}.kalman_R_ib_omega;
    end
    omega.mean(k) = mean(omega.ts{k});
end

P = 6; % Torque load
for k = 1:n
    if TRUNCATE
        T_L.ts{k} = S{P,k}.kalman_R_ib_T_L(end-L:end);
    else
        T_L.ts{k} = S{P,k}.kalman_R_ib_T_L;
    end
    T_L.mean(k) = mean(T_L.ts{k});
end

P = 7; % Resistance, EKF
for k = 1:n
    if TRUNCATE
        R.ts{k}   = S{P,k}.kalman_R_ib_R(end-L:end);
    else
        R.ts{k}   = S{P,k}.kalman_R_ib_R;
    end
    R.mean(k) = mean(R.ts{k});
    R.var(k)  = var(R.ts{k});
end

P = 15; % Temperature
for k = 1:n
    if TRUNCATE
        Tw.ts{k} = S{P,k}.kalman_R_ib_Tw(end-L:end);
    else
        Tw.ts{k} = S{P,k}.kalman_R_ib_Tw;
    end
    Tw.mean(k)= mean(Tw.ts{k});
end

Q = zeros(1,n);
Irms = zeros(1,n);
Vrms = zeros(1,n);
Rbar = zeros(1,n);
for k = 1:n
    Q(k) = mean(R.ts{k}.*(i.ts{k}.^2)); % Avg. Joule losses, <R*i^2>
    
    Irms(k) = i.rms(k); % RMS value of current, sqrt(<i^2>)
    
    Rbar(k) = Q(k)/(Irms(k)^2); % Resulting average resistance
    
    K = 56.6e-3;
    V = u.ts{k} - K*omega.ts{k};
    Vrms(k) = norm(V)/sqrt(length(V));
end

% vltg,crnt,vlct,rstnc,trq,tmprtr

vltg.points = Vrms;
vltg.mean = mean(Vrms);
vltg.std = std(Vrms);

crnt.points = Irms;
crnt.mean = mean(Irms);
crnt.std = std(Irms);

vlct.points = omega.mean;
vlct.mean = mean(omega.mean);

rstnc.points = Rbar;
rstnc.mean = mean(Rbar);
rstnc.std = std(Rbar);
rstnc.alt = mean(R.mean);

trq.points = T_L.mean;
trq.mean = mean(T_L.mean);

tmprtr.points = Tw.mean;
tmprtr.mean = mean(Tw.mean);
