L = 0.487/1000; % H
R = 2.74; % Ohm
K_T = 56.6/1000; % Nm/A
K_E = K_T; % V/(rad/s)
J = 67.8/(1000*100^2); % kg m^2

% Thermal model
% Celsius to Kelvin: K = C + 273.15 
% The states are Tw and Th - winding and housing temperatures.
% The inputs are Q and Tinf - Joule losses (electrical power loss) and
% ambient temperature.

%Tinf = 25.0 + 273.15; % Ambient temperature (approx. 25.0 C)

tau1 =   44.2; % s
tau2 = 1120.0; % s

Rth1 = 3.4; % K/W
Rth2 = 6.4; % K/W

C1 = tau1/Rth1;
C2 = tau2/Rth2;

E = diag([C1 C2]);

F = [-1/Rth1, 1/Rth1; 1/Rth1, -(1/Rth1 + 1/Rth2)]; 
G = [1, 0; 0, 1/Rth2];

A = inv(E)*F;
B = inv(E)*G;

C = [0, 1]; % Th (housing temperature) is measured
D = [0, 0];

% Kalman
G = [1; 1];
H = [0];

QN = 0.001;
RN = 0.001;

%Ts = 5e-4;

sys = ss(A, [B G], C, [D H]);
sysd = c2d(sys, Ts);

[KEST,l,P] = kalman(sysd, QN, RN);



