% Root locus for current feedback.

% Data for DC motor (Maxon RE 36.118800)
L = 0.487e-3; %mH -> H
R = 2.74; %Omega
K_T = 56.6e-3; %mNm/A -> Nm/A
K_E = K_T; %V/(rad/s)
J = 6.78e-6; %g cm^2 -> kg m^2

% Sampling time
Ts = 1.0e-3; % s

% Transfer function from voltage input to velocity output
sys = tf([J 0],[J*L J*R K_E*K_T]);

% Butterworth filter with cutoff fequency at 100 Hz
[numbutt,denbutt] = butter(2,2*pi*100,'s');
sys_butter = tf(numbutt, denbutt);

% Exact discretizations
% Motor
sysd = c2d(sys, Ts, 'zoh');

% Motor with PI controller
Ki = 1e3;
Ti = 1/Ki;
sys_pi = tf([Ti 1], [Ti 0]);
sysd_pi = c2d(sys*sys_pi, Ts, 'zoh');

% Motor in series with Butterworth filter
sysdd = c2d(sys*sys_butter, Ts, 'zoh');
sysdd_pi = c2d(sys*sys_butter*sys_pi, Ts, 'zoh');

% Unit delay on input
z = tf('z',Ts);
sysddd = 1/z*sysdd;
sysddd_pi = 1/z*sysdd_pi;

% Controller gains
K = 1:1:25;

% Root locus plot
switch 2
    case 1
        rlocus(sysd, 'r+-', sysdd, 'gx-', sysddd, 'b.-', K)
        a = 1.3;
        axis([-a a -a a])
        legend('Motor', 'Motor and Butterworth filter', ...
            'Motor, Butterworth filter and delay', 'Location', 'Best')
        title('Root locus, P controller, unit spacing for controller gains')
    case 2
        rlocus(sysd_pi, 'r+-', sysdd_pi, 'gx-', sysddd_pi, 'b.-', K)
        a = 1.3;
        axis([-a a -a a])
        legend('Motor', 'Motor and Butterworth filter', ...
            'Motor, Butterworth filter and delay', 'Location', 'Best')
        title('Root locus, PI controller, unit spacing for controller gains')
end
