% Performs nonlinear regression using Gauss-Newton iteration.
% Arnfinn Aas Eielsen, 2007

SEL = 1; % Select model (rational/exponential)

% Initial conditions
switch SEL
    case 1
        a = 10; b = 10; c = 10;
    case 2
        a = 5; b = -5; c = 5;
end
th = [a; b; c];

N = 150; % Number of iterations
for i=1:N
    a = th(1); b = th(2); c = th(3);
    
    switch SEL
        case 1
            num = b + c.*x;
            den = 1 + a.*x;
            y_hat = num ./ den;
            
            J = [(-(num.*x)./(den.^2))', (1./den)', (x./den)'];
        case 2
            y_hat = a*exp(b*x) + c;
            
            J = [(exp(b*x))', (a*x.*exp(b*x))', ones(length(x),1)];
    end
    
    r = (y - y_hat)';
    
    [Q,R] = qr(J);
    H = R'*R;
    if rcond(H) > eps
        p = H\J'*r;
        th = th + p;
    else
        disp('Jacobian is close to being rank-deficient, stopping.');
        break
    end
end

xx = min(x):0.01:max(x);
switch SEL
    case 1
        yy = (th(2) + th(3)*xx)./(1 + th(1)*xx);
    case 2
        yy = th(1)*exp(th(2)*xx) + th(3);
end

plot(x, y, '.', xx, yy)

