% Second order Pad approximant.
% Arnfinn Aas Eielsen, 2007
function [F,J] = residual_pade_vector(th)

% Assume the sample times and measurements are initialized somewhere.
global x y

if length(th) == 4
    a = th(1); b = th(2); c = th(3); d = th(4);
    num = 12*(a*exp(b*d) + c) + 6*(a*b*exp(b*d) - c*b).*(x-d) + ...
        (a*b^2*exp(b*d) + b^2*c).*(x-d).^2;
    den = 12 - 6*b.*(x-d) + b^2.*(x-d).^2;
elseif length(th) == 3
    a = th(1); b = th(2); c = th(3);
    num = 12*(a + c) + 6*(a*b - c*b).*x + (a*b^2 + b^2*c).*x.^2;
    den = 12 - 6*b.*x + b^2.*x.^2;
end

eta = num ./ den;

F = y - eta;

if nargout > 1
    if length(th) == 4
        J = [-( (12*exp(b*d) + 6*b*exp(b*d).*(x-d) +  ...
                b^2*exp(b*d).*(x-d).^2) ./ den )', ...
             -( ( (12*a*d*exp(b*d) + 6*(a*b*d*exp(b*d) + ...
                a*exp(b*d) - c).*(x-d) + ...
                (a*b^2*d*exp(b*d) + 2*a*b*exp(b*d) + 2*b*c).*(x-d).^2) .* den - ...
                num .* (-6.*(x-d) + 2*b.*(x-d).^2) ) ./ den.^2 )', ...
             -( (12 - 6*b.*(x-d) +  b^2.*(x-d).^2) ./ den )', ...
             -( ( (6*(a*b*exp(b*d) + b*c) + ...
                (4*a*b^2*exp(b*d) - 2*b^2*c).*(x-d) + ...
                a*b^3*exp(b*d).*(x-d).^2) .* den - ...
                num .* (6*b - 2*b^2.*(x-d)) ) ./ den.^2 )'];
    elseif length(th) == 3
        J = [-( (12 + 6*b.*x +  b^2.*x.^2) ./ den )', ...
             -( ( (6*(a - c).*x + 2*(a*b + b*c).*x.^2) .* den - ...
                num .* (-6.*x + 2*b.*x.^2) ) ./ den.^2 )', ...
             -( (12 - 6*b.*x +  b^2.*x.^2) ./ den )'];
    end
end
