% Runs lsqnonlin with various models.
% Arnfinn Aas Eielsen, 2007

clear sol

SEL = 5; % Select model

global sol
clear sol

if SEL == 1 % Use exponential model
    model_name = 'exponential model';
    th0 = [1,-1,1];
    sol(1,:) = lsqnonlin(@residual_exponential_vector,th0,[],[]);
    options = optimset('Jacobian','on');
    sol(2,:) = lsqnonlin(@residual_exponential_vector,th0,[],[],options);
    ar = sol(2,1); br = sol(2,2); cr = sol(2,3);
    xx = min(x):0.01:max(x);
    yy = ar*exp(br*xx)+cr;
    
elseif SEL == 2 % Use Pad approximant
    model_name = 'second order Pad approximant';
    th0 = [1,-1,1,1];
    sol(1,:) = lsqnonlin(@residual_pade_vector,th0,[],[]);
    options = optimset('Jacobian','on');
    sol(2,:) = lsqnonlin(@residual_pade_vector,th0,[],[],options);
    ar = sol(2,1); br = sol(2,2); cr = sol(2,3); dr = sol(2,4);
    xx = min(x):0.01:max(x);
    num_m = 12*(ar*exp(br*dr) + cr) + 6*(ar*br*exp(br*dr) - cr*br).*(xx-dr) + (ar*br^2*exp(br*dr) + br^2*cr).*(xx-dr).^2;
    den_m = 12 - 6*br.*(xx-dr) + br^2.*(xx-dr).^2;
    yy = num_m ./ den_m;
    
elseif SEL == 3 % Use Pad approximant with d = 0
    model_name = 'second order Pad approximant';
    th0 = [1,-1,1];
    sol(1,:) = lsqnonlin(@residual_pade_vector,th0,[],[]);
    options = optimset('Jacobian','on');
    sol(2,:) = lsqnonlin(@residual_pade_vector,th0,[],[],options);
    ar = sol(2,1); br = sol(2,2); cr = sol(2,3); dr = 0;
    xx = min(x):0.01:max(x);
    num_m = 12*(ar*exp(br*dr) + cr) + 6*(ar*br*exp(br*dr) - cr*br).*(xx-dr) + (ar*br^2*exp(br*dr) + br^2*cr).*(xx-dr).^2;
    den_m = 12 - 6*br.*(xx-dr) + br^2.*(xx-dr).^2;
    yy = num_m ./ den_m;
    
elseif SEL == 4 % Use Holliday model
    model_name = 'Holliday model';
    th0 = [1,1,1];
    sol(1,:) = lsqnonlin(@residual_holliday_vector,th0,[],[]);
    options = optimset('Jacobian','on','TolFun',1e-15,'TolX',1e-15);
    sol(2,:) = lsqnonlin(@residual_holliday_vector,th0,[],[],options);
    ar = sol(2,1); br = sol(2,2); cr = sol(2,3);
    xx = min(x):0.01:max(x);
    num = 1;
    den = ar + br.*xx + cr.*xx.^2;
    yy = num ./ den;
    
elseif SEL == 5 % Use rational model
    model_name = 'rational model';
    th0 = [1,1,1];
    sol(1,:) = lsqnonlin(@residual_rational_vector,th0,[],[]);
    options = optimset('Jacobian','on','MaxFunEvals',1e4,'MaxIter',1e4);
    sol(2,:) = lsqnonlin(@residual_rational_vector,th0,[],[],options);
    SEL = 2;
    ar = sol(SEL,1); br = sol(SEL,2); cr = sol(SEL,3);
    xx = min(x):0.01:max(x);
    yy = (br + cr*xx)./(1 + ar*xx);
    
elseif SEL == 6 % Use inverse cotanget model
    model_name = 'inverse cotanget model';
    th0 = [1,1,1];
    sol(1,:) = lsqnonlin(@residual_acot_vector,th0,[],[]);
    SEL = 1;
    ar = sol(SEL,1); br = sol(SEL,2); cr = sol(SEL,3);
    xx = min(x):0.01:max(x);
    yy = ar*acot(br*xx) + cr;
    
elseif SEL == 7 % Use Freundlich model
    model_name = 'Freundlich model';
    th0 = [1,1,1];
    sol(1,:) = lsqnonlin(@residual_freundlich_vector,th0,[],[]);
    SEL = 1;
    ar = sol(SEL,1); br = sol(SEL,2); cr = sol(SEL,3);
    xx = min(x):0.01:max(x);
    yy = ar + br*xx.^cr;
    
elseif SEL == 8 % Use Bleasdale Nelder model
    model_name = 'Bleasdale Nelder model';
    th0 = [1,1,1];
    sol(1,:) = lsqnonlin(@residual_bleasdale_nelder_vector,th0,[],[]);
    SEL = 1;
    ar = sol(SEL,1); br = sol(SEL,2); cr = sol(SEL,3);
    xx = min(x):0.01:max(x);
    yy = (ar + br*xx).^(1/cr);

elseif SEL == 9 % Use rational 2nd order model 5 parameters
    model_name = 'rational model';
    th0 = [1,1,1,1,1];
    sol(1,:) = lsqnonlin(@residual_rational_2_vector,th0,[],[]);
    options = optimset('Jacobian','on','MaxFunEvals',1e4,'MaxIter',1e4);
    sol(2,:) = lsqnonlin(@residual_rational_2_vector,th0,[],[],options);
    SEL = 2;
    ar = sol(SEL,1); br = sol(SEL,2); cr = sol(SEL,3); dr = sol(SEL,4); er = sol(SEL,5);
    xx = min(x):0.01:max(x);
    yy = (cr + dr.*xx + er.*xx.^2)./(1 + ar.*xx + br.*xx.^2);
    
elseif SEL == 10 % Use rational 2nd order model 3 parameters
    model_name = 'rational model';
    th0 = [1,1,1];
    sol(1,:) = lsqnonlin(@residual_rational_3_vector,th0,[],[]);
    options = optimset('Jacobian','on','MaxFunEvals',1e4,'MaxIter',1e4);
    sol(2,:) = lsqnonlin(@residual_rational_3_vector,th0,[],[],options);
    SEL = 2;
    ar = sol(SEL,1); br = sol(SEL,2); cr = sol(SEL,3);
    xx = min(x):0.01:max(x);
    yy = (br + cr.*xx.^2)./(1 + ar.*xx.^2);
end



disp(sol)

if 1
    plot(x, y, '.', xx, yy)
    %axis([0 2 0 4])
    xlabel('Estimated current as RMS [A]')
    ylabel('Estimated terminal resistance [\Omega]')
    legend('Dataset', ['Model fit, ' model_name], 'Location', 'Best')
end
