// Arnfinn Aas Eielsen

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int gauss(long double *, long double *, int);
void print_matrix(long double *, int, int);
void print_vector(long double *, int);
void matrixMultiply(long double *, long double *, int, int, int, long double *);
void matrixTranspose(long double *, int, int, long double *);
void matrixInverse(long double *, int, long double *);
void matrixCopy(long double *, int, int, long double *);

int main(void)
{
	if (0) {
		FILE *input = fopen("gauss_test.dat", "r");
		if (input == NULL) {
			printf("Feil: Finner ikke input-fil.\n");
			return -1;
		}

		int i, j, R, K;

		printf("\nAntall variable:  ");
		fscanf(input, "%d\n", &K);
		printf("%d\n\n", K);

		printf("Antall ligninger: ");
		fscanf(input, "%d\n", &R);
		printf("%d\n\n", R);

		if (K != R) {
			printf("Feil: matrisen er ikke kvadratisk.\n");
			return -1;
		}

		int N = K;
		int M = N+1;

		float A[N*N], b[N];
		long double *C = (long double *)malloc(sizeof(long double)*N*M);
		long double *x = (long double *)malloc(sizeof(long double)*N);

		printf("matrise A:\n\n");

		for(i = 0; i < N; i++)
		{
			for(j = 0; j < N; j++)
			{
				fscanf(input,"%f", &A[N*i+j]);
				printf("%8.2f", A[N*i+j]);
			}
			printf("\n");
		}

		printf("\nvektor b:\n\n");
		for(i = 0; i < N; i++)
		{
			fscanf(input, "%f\n", &b[i]);
			printf("%8.2f\n%", b[i]);
		}

		printf("\nligningssystem C:\n\n");
		for(i = 0; i < N; i++)
		{
			for(j = 0; j < N; j++)
			{
				C[M*i+j] = (long double)A[N*i+j];
				printf("%8.2f  ", (float)C[M*i+j]);
			}
			C[M*i+N] = (long double)b[i];
			printf("%8.2f  \n", (float)C[M*i+N]);
		}

		fclose(input);

		// --- Gauss algoritme ---
		if (gauss(C, x, N) == -1)
		{
			return -1;
		}

		printf("Lsning:\n");

		for (i=0; i < N; i++)
		{
			printf("x%d: %.3f\n", i, (float)x[i]);
		}

		free(C);
		free(x);
	} else {

		int NN = 4;
		int MM = 4;

		//long double G[9] = {8.0, 1.0, 6.0, 3.0, 5.0, 7.0, 4.0, 9.0, 2.0};
		//long double G[16] = {16.0, 2.0, 3.0, 13.0, 5.0, 11.0, 10.0, 8.0, 9.0, 7.0, 6.0, 12.0, 4.0, 14.0, 15.0, 1.0};
		long double G[16] = {1.0, 1.0, 1.0, 1.0, 1.0, 2.0, 3.0, 4.0, 1.0, 3.0, 6.0, 10.0, 1.0, 4.0, 10.0, 20.0};

		long double *L = (long double *)malloc(sizeof(long double)*NN*MM);

		printf("\nG:\n");
		print_matrix(G, NN, MM);

		printf("\n\n");

		matrixInverse(G, NN, L);

		printf("\nL:\n");
		print_matrix(L, NN, MM);

		free(L);
	}

	return 0;
}

// Multiply two matrices: A (n x o), B (o x m) -> C (n x m)
void matrixMultiply(long double *A, long double *B, int N, int O, int M, long double *C)
{
    int i, j, k;

    for (i = 0; i < N; i++) { // for all columns of A
        for (j = 0; j < M; j++) { // for all rows of B
            C[M*i+j] = 0.0;
            for (k = 0; k < O; k++) {
                C[M*i+j] = C[M*i+j] + A[O*i+k]*B[M*k+j];
            }
        }
    }
}

// Transpose a matrix: A (n x m) -> B (m x n)
void matrixTranspose(long double *A, int N, int M, long double *B)
{
    int i, j;

    for (i = 0; i < N; i++) {
        for (j = 0; j < M; j++) {
            B[N*j+i] = A[M*i+j];
        }
    }
}

// Inverse of a square matrix: A (n x n) -> A^-1 (n x n)
void matrixInverse(long double *A, int N, long double *X)
{

    int i, j, k;
    int M;

    M = N+1;

    long double *x = (long double *)malloc(sizeof(long double)*N);
    long double *C = (long double *)malloc(sizeof(long double)*N*M);
    //long double *AA = (long double *)malloc(sizeof(long double)*N*M);

    for(k = 0; k < N; k++) {

		printf("k: %d\n", k);

		for(i = 0; i < N; i++) {
			for(j = 0; j < N; j++) {
				C[M*i+j] = A[N*i+j];
				printf("%8.2f  \n", (float)C[M*i+j]);
			}
    	}

        for(i = 0; i < N; i++) {
            C[M*i+N] = 0.0;
            printf("%8.2f  \n", (float)C[M*i+N]);
        }
        C[M*k+N] = 1.0;
        printf("%8.2f  \n", (float)C[M*k+N]);

        gauss(C, x, N);

        for(i = 0; i < N; i++) {
            X[N*i+k] = x[i];
            printf("%8.2f  \n", (float)X[N*i+k]);
        }
    }

    free(x);
    free(C);
}

void matrixCopy(long double *A, int N, int M, long double *B)
{
    int i, j;

    for (i = 0; i < N; i++) {
        for (j = 0; j < M; j++) {
            B[M*i+j] = A[M*i+j];
        }
    }
}
