#include <math.h>
#include <string.h>
#include "test.h"
#include "test_prm.h"

void MdlInitialize(void)
{
  
  {
    SimStruct *rts = ssGetSFunction(rtS, 0);
    sfcnInitializeConditions(rts);
  }
  
  rtDWork.Discrete_State_Space_DSTATE[0] = rtP.Discrete_State_Space_X0[0];
  rtDWork.Discrete_State_Space_DSTATE[1] = rtP.Discrete_State_Space_X0[1];

  rtDWork.Difference_DSTATE = 0.0;
}

void MdlStart(void)
{
  
  {
    MQ_adcInitialize(0x320);
  }

  rtB.u_bias = rtP.u_bias_Value;

  {
    int16_T value;

    value = MQ_dacVoltageToOutput(rtP.Analog_Output_P2);
    MQ_dacOutput(0x320, (uint_T)rtP.Analog_Output_P1, value);
  }

  {
    MQ3_encResetAll(0x320);
    MQ3_encSetCount(0x320, (uint_T)rtP.Encoder_Input_P1,
     (int_T)rtP.Encoder_Input_P2);
  }

  rtB.Q = rtP.Q_Value;

  rtB.Ambient = rtP.Ambient_Value;

  rtB.A_D_offset_Volt_a = rtP.A_D_offset_Volt_a_Value;

  rtB.Closed_loop_gain = rtP.Closed_loop_gain_Value;

  rtB.Input_offset_Volt = rtP.Input_offset_Volt_Value;

  rtB.T_offset_K = rtP.T_offset_K_Value;

  rtB.Kelvin = rtP.Kelvin_Value;

  rtB.A_D_offset_Volt_b = rtP.A_D_offset_Volt_b_Value;

  MdlInitialize();
}

void MdlOutputs(int_T tid)
{

  if (ssIsSampleHit(rtS, 1, tid)) {
    
    {
      int_T value;

      value = MQ_adcInput(0x320, (uint_T)rtP.Analog_Input_P1);
      rtB.Analog_Input = MQ_adcInputToVoltage(value);
    }

    {
      int_T value;

      value = MQ_adcInput(0x320, (uint_T)rtP.Current_filtered_P1);
      rtB.Current_filtered = MQ_adcInputToVoltage(value);
    }

    {
      int_T value;

      value = MQ_adcInput(0x320, (uint_T)rtP.Current_unfiltered_P1);
      rtB.Current_unfiltered = MQ_adcInputToVoltage(value);
    }

    {
      int_T value;

      value = MQ_adcInput(0x320, (uint_T)rtP.Analog_Input1_P1);
      rtB.Analog_Input1 = MQ_adcInputToVoltage(value);
    }
  }

  if (ssIsContinuousTask(rtS, tid)) {

    rtB.Sine_Wave = rtP.Sine_Wave_Amp *
      sin(rtP.Sine_Wave_Freq * ssGetTaskTime(rtS,tid) + rtP.Sine_Wave_Phase) +
      rtP.Sine_Wave_Bias;
  }

  if (ssIsSampleHit(rtS, 1, tid)) {
    rtB.u_bias = rtP.u_bias_Value;
  }

  if (ssIsContinuousTask(rtS, tid)) {
    rtB.Sum_a = rtB.Sine_Wave - rtB.u_bias;

    {
      int16_T value;

      value = MQ_dacVoltageToOutput(rtB.Sum_a);
      MQ_dacOutput(0x320, (uint_T)rtP.Analog_Output_P1, value);
    }
  }

  if (ssIsSampleHit(rtS, 1, tid)) {
    
    {

      rtB.Encoder_Input = MQ3_encInput(0x320, (uint_T)rtP.Encoder_Input_P1);
    }

    rtB.Ki_filt = rtB.Current_filtered * rtP.Ki_filt_Gain;
  }

  if (ssIsSampleHit(rtS, 2, tid)) {
    
    {
      SimStruct *rts = ssGetSFunction(rtS, 0);
      sfcnOutputs(rts, tid);
    }
  }

  if (ssIsSampleHit(rtS, 1, tid)) {
    rtB.Ki = rtB.Current_unfiltered * rtP.Ki_Gain;

    rtB.Sum1_a[0] = rtB.Ki - rtB.S_Function;
    rtB.Sum1_a[1] = rtB.Ki_filt - rtB.S_Function;

    rtB.Q = rtP.Q_Value;

    rtB.Ambient = rtP.Ambient_Value;

    rtB.A_A = rtB.Ki * rtP.A_A_Gain;

    rtB.A_D_offset_Volt_a = rtP.A_D_offset_Volt_a_Value;

    rtB.Sum1_b = - rtB.A_D_offset_Volt_a + rtB.Analog_Input;

    rtB.Closed_loop_gain = rtP.Closed_loop_gain_Value;

    rtB.Resiprocal = 1.0/(rtB.Closed_loop_gain);

    rtB.Product = rtB.Sum1_b * rtB.Resiprocal;

    rtB.Input_offset_Volt = rtP.Input_offset_Volt_Value;

    rtB.Sum_b = rtB.Product - rtB.Input_offset_Volt;

    rtB.V_to_mV = rtB.Sum_b * rtP.V_to_mV_Gain;

    rtB.Sum6 = - rtB.A_A + rtB.V_to_mV;

    rtB.Type_K_mV_to_Celsius = rt_Lookup(rtP.Type_K_mV_to_Celsius_XData, 207,
      rtB.Sum6, rtP.Type_K_mV_to_Celsius_YData);

    rtB.T_offset_K = rtP.T_offset_K_Value;

    rtB.Sum4 = rtB.Type_K_mV_to_Celsius - rtB.T_offset_K;

    rtB.Kelvin = rtP.Kelvin_Value;

    rtB.Sum2 = rtB.Sum4 + rtB.Kelvin;

    {
      rtB.Discrete_State_Space[0] = rtP.Discrete_State_Space_D[0]*rtB.Sum2;
      rtB.Discrete_State_Space[0] +=
        rtP.Discrete_State_Space_C[0]*rtDWork.Discrete_State_Space_DSTATE[1];

      rtB.Discrete_State_Space[1] = rtP.Discrete_State_Space_D[1]*rtB.Sum2;
      rtB.Discrete_State_Space[1] +=
        rtP.Discrete_State_Space_C[1]*rtDWork.Discrete_State_Space_DSTATE[0]
        +rtP.Discrete_State_Space_C[2]*rtDWork.Discrete_State_Space_DSTATE[1];

      rtB.Discrete_State_Space[2] = rtP.Discrete_State_Space_D[2]*rtB.Sum2;
      rtB.Discrete_State_Space[2] +=
        rtP.Discrete_State_Space_C[3]*rtDWork.Discrete_State_Space_DSTATE[1];
    }

    rtB.Sum3[0] = rtB.Discrete_State_Space[0] - rtB.Kelvin;
    rtB.Sum3[1] = rtB.Discrete_State_Space[1] - rtB.Kelvin;
    rtB.Sum3[2] = rtB.Discrete_State_Space[2] - rtB.Kelvin;

    rtB.Type_T_mV_to_Celsius = rt_Lookup(rtP.Type_T_mV_to_Celsius_XData, 209,
      rtB.Sum6, rtP.Type_T_mV_to_Celsius_YData);

    rtB.A_D_offset_Volt_b = rtP.A_D_offset_Volt_b_Value;

    rtB.Sum5 = - rtB.A_D_offset_Volt_b + rtB.Analog_Input1;

    rtB.rad_count = rtB.Encoder_Input * rtP.rad_count_Gain;

    rtB.Difference = rtP.Difference_D*rtB.rad_count;
    rtB.Difference += rtP.Difference_C*rtDWork.Difference_DSTATE;

    rtB.Ts = rtB.Difference * rtP.Ts_Gain;
  }
}

void MdlUpdate(int_T tid)
{
  if (ssIsSampleHit(rtS, 1, tid)) {
    
    {
      static real_T xnew[2];
      xnew[0] =
        rtP.Discrete_State_Space_B[0]*rtB.Q+rtP.Discrete_State_Space_B[1]*rtB.Ambient
        +rtP.Discrete_State_Space_B[2]*rtB.Sum2;
      xnew[0] +=
        rtP.Discrete_State_Space_A[0]*rtDWork.Discrete_State_Space_DSTATE[0]
        +rtP.Discrete_State_Space_A[1]*rtDWork.Discrete_State_Space_DSTATE[1];

      xnew[1] =
        rtP.Discrete_State_Space_B[3]*rtB.Q+rtP.Discrete_State_Space_B[4]*rtB.Ambient
        +rtP.Discrete_State_Space_B[5]*rtB.Sum2;
      xnew[1] +=
        rtP.Discrete_State_Space_A[2]*rtDWork.Discrete_State_Space_DSTATE[0]
        +rtP.Discrete_State_Space_A[3]*rtDWork.Discrete_State_Space_DSTATE[1];
      (void)memcpy(&rtDWork.Discrete_State_Space_DSTATE[0], xnew,
       sizeof(real_T)*2);
    }

    {
      rtDWork.Difference_DSTATE = rtB.rad_count
        +rtP.Difference_A*rtDWork.Difference_DSTATE;
    }
  }
}

void MdlTerminate(void)
{
  
  {
    int16_T value;

    value = MQ_dacVoltageToOutput(rtP.Analog_Output_P3);
    MQ_dacOutput(0x320, (uint_T)rtP.Analog_Output_P1, value);
  }

  {
    SimStruct *rts = ssGetSFunction(rtS, 0);
    sfcnTerminate(rts);
  }
}

#include "test_reg.h"

#include "WinConLibrary.h"

void MdlRegister( void )
{
  
  {
    WCRegisterVariable("test\vCurrent", &rtB.Ki_filt);
    WCRegisterScope("test\vCurrent", 1
     , &rtB.Ki_filt
     );
  }

  {
    WCRegisterVariable("test\vCurrent bias", &rtB.S_Function);
    WCRegisterScope("test\vCurrent bias", 1
     , &rtB.S_Function
     );
  }

  {
    WCRegisterVariable("test\vCurrent bias disp.", &rtB.S_Function);
    WCRegisterDisplay("test\vCurrent bias disp.", 1
     , &rtB.S_Function
     );
  }

  {
    WCRegisterVariable("test\vCurrent scope[0]", &rtB.Ki);
    WCRegisterVariable("test\vCurrent scope[1]", &rtB.Ki_filt);
    WCRegisterVariable("test\vCurrent scope[2]", &rtB.Sum1_a[0]);
    WCRegisterVariable("test\vCurrent scope[3]", &rtB.Sum1_a[1]);
    WCRegisterScope("test\vCurrent scope", 4
     , &rtB.Ki
     , &rtB.Ki_filt
     , &rtB.Sum1_a[0]
     , &rtB.Sum1_a[1]
     );
  }

  {
    WCRegisterVariable("test\vTemp.", &rtB.Sum3[0]);
    WCRegisterScope("test\vTemp.", 1
     , &rtB.Sum3[0]
     );
  }

  {
    WCRegisterVariable("test\vType K", &rtB.Type_K_mV_to_Celsius);
    WCRegisterScope("test\vType K", 1
     , &rtB.Type_K_mV_to_Celsius
     );
  }

  {
    WCRegisterVariable("test\vType K mod.", &rtB.Sum4);
    WCRegisterScope("test\vType K mod.", 1
     , &rtB.Sum4
     );
  }

  {
    WCRegisterVariable("test\vType T", &rtB.Type_T_mV_to_Celsius);
    WCRegisterScope("test\vType T", 1
     , &rtB.Type_T_mV_to_Celsius
     );
  }

  {
    WCRegisterVariable("test\vCh. 3", &rtB.Sum5);
    WCRegisterScope("test\vCh. 3", 1
     , &rtB.Sum5
     );
  }

  {
    WCRegisterVariable("test\vPosition", &rtB.rad_count);
    WCRegisterScope("test\vPosition", 1
     , &rtB.rad_count
     );
  }

  {
    WCRegisterVariable("test\vVelocity", &rtB.Ts);
    WCRegisterScope("test\vVelocity", 1
     , &rtB.Ts
     );
  }

  {
    WCRegisterVariable("test\vAnalog Input", &rtB.Analog_Input);
    WCRegisterVariable("test\vCurrent filtered", &rtB.Current_filtered);
    WCRegisterVariable("test\vCurrent unfiltered", &rtB.Current_unfiltered);
    WCRegisterVariable("test\vAnalog Input1", &rtB.Analog_Input1);
    WCRegisterVariable("test\vSine Wave", &rtB.Sine_Wave);
    WCRegisterVariable("test\vu_bias", &rtB.u_bias);
    WCRegisterVariable("test\vSum", &rtB.Sum_a);
    WCRegisterVariable("test\vEncoder Input", &rtB.Encoder_Input);
    WCRegisterVariable("test\vKi_filt", &rtB.Ki_filt);
    WCRegisterVariable("test\vS-Function", &rtB.S_Function);
    WCRegisterVariable("test\vKi", &rtB.Ki);
    WCRegisterVariable("test\vSum1[0]", &rtB.Sum1_a[0] + 0);
    WCRegisterVariable("test\vSum1[1]", &rtB.Sum1_a[0] + 1);
    WCRegisterVariable("test\vQ", &rtB.Q);
    WCRegisterVariable("test\vAmbient", &rtB.Ambient);
    WCRegisterVariable("test\vA//A", &rtB.A_A);
    WCRegisterVariable("test\vAD595\vA//D offset [Volt]",
     &rtB.A_D_offset_Volt_a);
    WCRegisterVariable("test\vAD595\vSum1", &rtB.Sum1_b);
    WCRegisterVariable("test\vAD595\vClosed loop gain", &rtB.Closed_loop_gain);
    WCRegisterVariable("test\vAD595\vResiprocal", &rtB.Resiprocal);
    WCRegisterVariable("test\vAD595\vProduct", &rtB.Product);
    WCRegisterVariable("test\vAD595\vInput offset [Volt]",
     &rtB.Input_offset_Volt);
    WCRegisterVariable("test\vAD595\vSum", &rtB.Sum_b);
    WCRegisterVariable("test\vAD595\v[V] to [mV]", &rtB.V_to_mV);
    WCRegisterVariable("test\vSum6", &rtB.Sum6);
    WCRegisterVariable("test\vType K mV to Celsius", &rtB.Type_K_mV_to_Celsius);
    WCRegisterVariable("test\vT offset [K]", &rtB.T_offset_K);
    WCRegisterVariable("test\vSum4", &rtB.Sum4);
    WCRegisterVariable("test\vKelvin", &rtB.Kelvin);
    WCRegisterVariable("test\vSum2", &rtB.Sum2);
    WCRegisterVariable("test\vDiscrete State-Space[0]",
     &rtB.Discrete_State_Space[0] + 0);
    WCRegisterVariable("test\vDiscrete State-Space[1]",
     &rtB.Discrete_State_Space[0] + 1);
    WCRegisterVariable("test\vDiscrete State-Space[2]",
     &rtB.Discrete_State_Space[0] + 2);
    WCRegisterVariable("test\vSum3[0]", &rtB.Sum3[0] + 0);
    WCRegisterVariable("test\vSum3[1]", &rtB.Sum3[0] + 1);
    WCRegisterVariable("test\vSum3[2]", &rtB.Sum3[0] + 2);
    WCRegisterVariable("test\vType T mV to Celsius", &rtB.Type_T_mV_to_Celsius);
    WCRegisterVariable("test\vA//D offset [Volt]", &rtB.A_D_offset_Volt_b);
    WCRegisterVariable("test\vSum5", &rtB.Sum5);
    WCRegisterVariable("test\vrad//count", &rtB.rad_count);
    WCRegisterVariable("test\vDifference", &rtB.Difference);
    WCRegisterVariable("test\vTs", &rtB.Ts);
  }

  {
    WCRegisterScalarParameter("test\vAnalog Input\vChannels",
     &rtP.Analog_Input_P1);

    WCRegisterScalarParameter("test\vCurrent filtered\vChannels",
     &rtP.Current_filtered_P1);

    WCRegisterScalarParameter("test\vCurrent unfiltered\vChannels",
     &rtP.Current_unfiltered_P1);

    WCRegisterScalarParameter("test\vAnalog Input1\vChannels",
     &rtP.Analog_Input1_P1);

    WCRegisterScalarParameter("test\vSine Wave\vAmplitude", &rtP.Sine_Wave_Amp);

    WCRegisterScalarParameter("test\vSine Wave\vBias", &rtP.Sine_Wave_Bias);

    WCRegisterScalarParameter("test\vSine Wave\vFrequency", &rtP.Sine_Wave_Freq);

    WCRegisterScalarParameter("test\vSine Wave\vPhase", &rtP.Sine_Wave_Phase);

    WCRegisterScalarParameter("test\vu_bias\vValue", &rtP.u_bias_Value);

    WCRegisterScalarParameter("test\vAnalog Output\vChannels",
     &rtP.Analog_Output_P1);

    WCRegisterScalarParameter("test\vAnalog Output\vInitialOutputs",
     &rtP.Analog_Output_P2);

    WCRegisterScalarParameter("test\vAnalog Output\vFinalOutputs",
     &rtP.Analog_Output_P3);

    WCRegisterScalarParameter("test\vEncoder Input\vChannels",
     &rtP.Encoder_Input_P1);

    WCRegisterScalarParameter("test\vEncoder Input\vInitialValues",
     &rtP.Encoder_Input_P2);

    WCRegisterScalarParameter("test\vKi_filt\vGain", &rtP.Ki_filt_Gain);

    WCRegisterScalarParameter("test\vKi\vGain", &rtP.Ki_Gain);

    WCRegisterScalarParameter("test\vQ\vValue", &rtP.Q_Value);

    WCRegisterScalarParameter("test\vAmbient\vValue", &rtP.Ambient_Value);

    WCRegisterScalarParameter("test\vA//A\vGain", &rtP.A_A_Gain);

    WCRegisterScalarParameter("test\vAD595\vA//D offset [Volt]\vValue",
     &rtP.A_D_offset_Volt_a_Value);

    WCRegisterScalarParameter("test\vAD595\vClosed loop gain\vValue",
     &rtP.Closed_loop_gain_Value);

    WCRegisterScalarParameter("test\vAD595\vInput offset [Volt]\vValue",
     &rtP.Input_offset_Volt_Value);

    WCRegisterScalarParameter("test\vAD595\v[V] to [mV]\vGain",
     &rtP.V_to_mV_Gain);

    WCRegisterVectorParameter("test\vType K mV to Celsius\vInputValues", 207,
     &rtP.Type_K_mV_to_Celsius_XData[0]);

    WCRegisterVectorParameter("test\vType K mV to Celsius\vOutputValues", 207,
     &rtP.Type_K_mV_to_Celsius_YData[0]);

    WCRegisterScalarParameter("test\vT offset [K]\vValue",
     &rtP.T_offset_K_Value);

    WCRegisterScalarParameter("test\vKelvin\vValue", &rtP.Kelvin_Value);

    WCRegisterVectorParameter("test\vDiscrete State-Space\vX0", 2,
     &rtP.Discrete_State_Space_X0[0]);

    WCRegisterVectorParameter("test\vType T mV to Celsius\vInputValues", 209,
     &rtP.Type_T_mV_to_Celsius_XData[0]);

    WCRegisterVectorParameter("test\vType T mV to Celsius\vOutputValues", 209,
     &rtP.Type_T_mV_to_Celsius_YData[0]);

    WCRegisterScalarParameter("test\vA//D offset [Volt]\vValue",
     &rtP.A_D_offset_Volt_b_Value);

    WCRegisterScalarParameter("test\vrad//count\vGain", &rtP.rad_count_Gain);

    WCRegisterScalarParameter("test\vTs\vGain", &rtP.Ts_Gain);
  }
}

void MdlChecksum( uint_T iValues[4] )
{

  iValues[0] = 1788878396U;
  iValues[1] = 810867731U;
  iValues[2] = 2777645723U;
  iValues[3] = 1717988265U;
}

uint_T MdlGetNumDataTypes()
{
  return 13;
}

boolean_T MdlGetDataTypeSizes(uint_T *puSizes, uint_T cDataTypes)
{
  
  
  
  

  if (cDataTypes == 13)
  {
    memcpy(puSizes, rtDataTypeSizes, cDataTypes * sizeof(uint_T));
    return 1; 
  }

  return 0; 
}

real_T MdlSamplingPeriod( void )
{
  return(0.0005);
}

