# Copyright (C) 1999 Quanser Consulting, Inc.
# All Rights Reserved.          
#
# File    : test.mk $Revision: 1.1.1.1 $
# Abstract:
#       Real-Time Workshop template makefile for building a WindowsNT-based 
#       real-time version of the SIMULINK model for WinCon.
#
#       The following defines can be used to modify the behavior of the
#       build:
#         OPTS           - User specific options, such as OPTS=-DMULTITASKING
#                          to enable multitasking mode.
#         OPT_OPTS       - Optimization option. Default is -oaxt. To enable
#                          debugging specify as OPT_OPTS=-d2.
#         USER_OBJS      - Additional user objects, such as files needed by
#                          S-functions.
#         USER_PATH      - The directory path to the source (.c) files which 
#                          are used to create any .obj files specified in 
#                          USER_OBJS. Multiple paths must be seperated
#                          with a semicolon. For example:
#                          USER_PATH="path1;path2"
#         USER_INCLUDES  - Additional include paths 
#                          (i.e. USER_INCLUDES="-Iwhere-ever -Iwhere-ever2")


#------------------------ Macros read by make_rtw -----------------------------
#
# The following macros are read by the Real-Time Workshop build procedure:
#       MAKE    - This is the command used to invoke the make utility
#       HOST    - What platform this template makefile is targeted for
#                 (i.e. PC or UNIX)
#       BUILD  - Invoke make from the Real-Time Workshop build procedure
#                (yes/no)?
#       TARGET - Type of target Real-Time (RT) or Nonreal-Tie (NRT)
#
MAKE            = nmake
HOST            = PC
BUILD           = yes
SYS_TARGET_FILE = wincon.tlc

#---------------------- Tokens expanded by make_rtw ---------------------------
#
# The following tokens, when wrapped with "|>" and "|<" are expanded by the 
# Real-Time Workshop build procedure.
#   
#  MODEL_NAME        - Name of the SIMULINK block diagram
#  MODEL_MODULES_OBJ - Object file name for any additional generated source 
#                      modules
#  MAKEFILE_NAME     - Name of makefile created from template makefile 
#                      <model>.mk
#  MATLAB_ROOT       - Path to were MATLAB is installed. 
#  MATLAB_BIN        - Path to MATLAB executable.
#  S_FUNCTIONS_OBJ   - List of S-functions .obj sources
#  SOLVER_OBJ        - Solver object file name
#  NUMST             - Number of sample times
#  TID01EQ           - yes (1) or no (0): Are sampling rates of continuous 
#                      task (tid=0) and 1st discrete task equal.
#  NCSTATES          - Number of continuous states
#  BUILDARGS         - Options passed in at the command line.

MODEL       = test
MODULES_OBJ = 
MAKEFILE    = test.mk
MATLAB_ROOT = C:\MATLAB6p1
MATLAB_BIN  = C:\MATLAB6p1\bin\win32
S_FUNCTIONS = bias.obj 
SOLVER      = 
NUMST       = 3
TID01EQ     = 1
NCSTATES    = 0
BUILDARGS   = 

#--------------------------------- Tool Locations -----------------------------
#
# Modify the following macro to reflect where you have installed WinCon.
#
WINCON = C:\PROGRA~1\Quanser\WINCON~1.3
WINCON_LIBS = $(WINCON)\lib
RTX = $(WINCON)\RTX\5.1

# Modify the following macro to reflect where you have installed
# the Microsoft C/C++ Compiler.
#
MSVCDIR = C:\PROGRA~1\MICROS~2\VC98
!ifndef MSVCDIR
!error MSVCDIR environment variable must be defined
!endif

MSDEVDIR = C:\PROGRA~1\MICROS~2
!ifndef MSDEVDIR
!error MSDEVDIR environment variable must be defined
!endif

MSBINDIR = C:\PROGRA~1\MICROS~2\COMMON\MSDEV98\BIN
!ifndef MSBINDIR
!error MSBINDIR environment variable must be defined
!endif

#---------------------------- Tool Definitions ---------------------------

CC     = "$(MSVCDIR)\bin\cl.exe"
LD     = "$(MSVCDIR)\bin\link.exe"
LIB    = "$(MSVCDIR)\lib"
LIBCMD = "$(MSVCDIR)\bin\lib.exe"

PERL   = "$(MATLAB_BIN)\perl"
MSGFLT = "$(WINCON)\bin\MsgFiltr"
NUL    = nul

#---------------------------- RTX Definitions ---------------------------

RTX_LIBS	= "$(RTX)"
RTX_STDLIBS = "$(RTX_LIBS)\rtxlibcmt.lib"

#------------------------ External mode ---------------------------------------
# uncomment -DVERBOSE to have information printed to stdout
!ifdef EXT_MODE
EXT_OBJ     =
EXT_CC_OPTS = /D "EXT_MODE" #/D "VERBOSE"
!else
EXT_OBJ     =
EXT_CC_OPTS =
!endif

#------------------------------ Include Path -----------------------------

MATLAB_INCLUDES = \
 "/I$(MATLAB_ROOT)\simulink\include" \
 "/I$(MATLAB_ROOT)\extern\include" \
 "/I$(MATLAB_ROOT)\rtw\c\src" \
 "/I$(MATLAB_ROOT)\rtw\c\libsrc" \
 "/I$(MATLAB_ROOT)\rtw\c\WinCon" \
 "/I$(MATLAB_ROOT)\rtw\c\dos\devices" \
 "/I$(MATLAB_ROOT)\rtw\c\dos\rti" \
 "/I$(MATLAB_ROOT)\toolbox\dspblks\src\rt" \
 "/I$(MATLAB_ROOT)\toolbox\dspblks\src\sim"

WINCON_INCLUDES = "/I$(WINCON)\include"

COMPILER_INCLUDES = "/I$(MSVCDIR)\include"

INCLUDES = $(WINCON_INCLUDES) $(MATLAB_INCLUDES) $(COMPILER_INCLUDES) $(USER_INCLUDES)

#-------------------------------- C Flags --------------------------------

# Required Options
REQ_OPTS = /nologo /ML /W3 /GX /D "WIN32" /D "NDEBUG" /D "_CONSOLE" /D "_MBCS" /YX /FD /c 

# Optimization Options.
#   /O2   : maximum optimization
#   /d2   : debugging options
#
OPT_OPTS = /O2

# Listing Options
#   /Fa   : assembly listing (put in .COD file)
#   /FAcs : source code intermixed with assembler

LIST_OPTS =

CC_OPTS = $(REQ_OPTS) $(OPT_OPTS) $(LIST_OPTS) $(OPTS) $(EXT_CC_OPTS)

CPP_REQ_DEFINES = /D "MODEL=$(MODEL)" /D "RT" /D "NUMST=$(NUMST)" \
                  /D "TID01EQ=$(TID01EQ)" /D "NCSTATES=$(NCSTATES)"

CFLAGS = $(CC_OPTS) $(CPP_REQ_DEFINES) $(INCLUDES)

#-------------------------------- Linker Flags --------------------------------

LD_OPTS = /NODEFAULTLIB /INCREMENTAL:NO /PDB:NONE /NOLOGO \
 /driver /align:0x20 /subsystem:native,4.00 /RELEASE /entry:_RtapiProcessEntryCRT@8
LDFLAGS = $(LD_OPTS)

#------------------------------- Source Files ---------------------------------

REQ_OBJS  = "$(RTX_LIBS)\startupCRT.obj" $(MODEL).obj $(MODULES_OBJ) rt_code.obj \
			 rt_sim.obj "$(WINCON_LIBS)\rtx_main.obj"
USER_OBJS =

OBJS = $(REQ_OBJS) $(USER_OBJS) $(S_FUNCTIONS) $(SOLVER) $(EXT_OBJ)

RTWLIB  = rtwlib.lib
WCLIB	= "$(WINCON_LIBS)\RTXUtil.lib" "$(WINCON_LIBS)\RTXLibrary.lib" $(RTX_STDLIBS) \
		  "$(MSVCDIR)\lib\oldnames.lib" "$(RTX_LIBS)\rtapi_rtss.lib" "$(RTX_LIBS)\rtx_rtss.lib"
LIBSRC  = "$(MATLAB_ROOT)\rtw\c\libsrc"
RTWOBJS =

#-------------------------- Source Path ---------------------------------------

# User source path

!ifdef USER_PATH
EXTRA_PATH = "$(USER_PATH)"
!else
EXTRA_PATH = 
!endif

#----------------------- Exported Environment Variables -----------------------
#
# Because of the 128 character command line length limitations in DOS, we
# use environment variables to pass additional information to the Microsoft Visual C++
# Compiler and Linker
#
PATH = $(MATLAB_ROOT)\bin\nt;$(MSVCDIR)\bin;$(MSDEVDIR)\bin;$(MSBINDIR)

# Make sure version 1.62 or higher of NMAKE is being used
#
!ifndef _NMAKE_VER
!error NMAKE version 1.62 or higher must be used.
!endif

#--------------------------------- Rules --------------------------------------

{$(MATLAB_ROOT)\rtw\c\src}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\rtw\c\libsrc}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\toolbox\comm\commsfun}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\toolbox\dspblks\src\sim}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\toolbox\dspblks\src\rt}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\toolbox\dspblks\dspmex}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\toolbox\fixpoint}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\toolbox\fuzzy\fuzzy\src}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(EXTRA_PATH)}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\rtw\c\WinCon}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{..}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\rtw\c\WinCon}.cpp.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\rtw\c\Quanser\Devices}.cpp.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{$(MATLAB_ROOT)\rtw\c\Quanser\Devices}.c.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

{..}.cpp.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

.cpp.obj::
        @$(CC) @<<$(MODEL).rsp
        $(CFLAGS) $<
<<

..\$(MODEL).wcl : $(OBJS) $(RTWLIB)
        @rem @<<$(MODEL).lnk
        $(LDFLAGS) $(OBJS) $(RTWLIB) $(WCLIB) /out:$@
<<KEEP
		@$(LD) @$(MODEL).lnk
		@del $(MODEL).lnk
        @echo ### Created executable: $(MODEL).wcl

# Build the library by creating a makefile and then executing it.

$(RTWLIB): rtw_proj.tmw $(RTWOBJS)
!if !exist(rtwlib)
	@mkdir rtwlib
!endif
	@echo >$(NUL) <<$(MODEL).rsp
		$(CFLAGS)
<<
	@dir /s /b $(LIBSRC)\*.c >> $(MODEL).rsp
	@echo ### Compiling library files:
	@$(CC) /Fortwlib\ @$(MODEL).rsp 
	@echo ### Creating library: $@
	@$(LIBCMD) /nologo /out:$@ rtwlib\*.obj $(RTWOBJS)
	@del rtwlib\*.obj
	-@rd rtwlib

#----------------------------- Dependencies -------------------------------

$(OBJS) : $(MAKEFILE) rtw_proj.tmw
